<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class SlaAvailibilityStatusLogs
 *
 * Created via:
 * oitc migrations create -p SLAModule SlaAvailibilityStatusLogs
 *
 * Run migration:
 * oitc migrations migrate -p SLAModule
 *
 */
class SlaAvailibilityStatusLogs extends AbstractMigration {
    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function up(): void {
        if (!$this->hasTable('sla_availability_status_hosts')) {
            $this->table('sla_availability_status_hosts')
                ->addColumn('id', 'biginteger', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('total_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Total time of the evaluation period 24h - 48h - 72h - 96h'
                ])
                ->addColumn('minimal_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Guaranteed available as a percentage'
                ])
                ->addColumn('determined_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Actual availability that was calculated by the SLA as percentage'
                ])
                ->addColumn('minimal_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Guaranteed available in seconds'
                ])
                ->addColumn('determined_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual availability that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_outage_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual outages that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_number_outages', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Number of outages'
                ])
                ->addColumn('up', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Up status in seconds'
                ])
                ->addColumn('down', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Down status in seconds'
                ])
                ->addColumn('unreachable', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Unreachable status in seconds'
                ])
                ->addColumn('evaluation_start', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Timestamp when the evaluation what started 01.03.2022 00:00:00'
                ])
                ->addColumn('evaluation_end', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Time when the evaluation what ended 01.03.2022 24:00:00'
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('updated', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'End date of the current evaluation process 01.03.2022 24:00:00'
                ])
                ->addIndex(
                    [
                        'host_id',
                        'sla_id',
                        'updated',
                    ]
                )
                ->addIndex(
                    [
                        'host_id',
                    ],
                    [
                        'unique' => true
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('sla_availability_status_services')) {
            $this->table('sla_availability_status_services')
                ->addColumn('id', 'biginteger', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('service_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('total_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Total time of the evaluation period 24h - 48h - 72h - 96h'
                ])
                ->addColumn('minimal_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Guaranteed available as a percentage'
                ])
                ->addColumn('determined_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Actual availability that was calculated by the SLA as percentage'
                ])
                ->addColumn('minimal_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Guaranteed available in seconds'
                ])
                ->addColumn('determined_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual availability that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_outage_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual outages that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_number_outages', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Number of outages'
                ])
                ->addColumn('ok', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Ok status in seconds'
                ])
                ->addColumn('warning', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Warning status in seconds'
                ])
                ->addColumn('critical', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Critical status in seconds'
                ])
                ->addColumn('unknown', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Unknown status in seconds'
                ])
                ->addColumn('evaluation_start', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Timestamp when the evaluation what started 01.03.2022 00:00:00'
                ])
                ->addColumn('evaluation_end', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Time when the evaluation what ended 01.03.2022 24:00:00'
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('updated', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'End date of the current evaluation process 01.03.2022 24:00:00'
                ])
                ->addIndex(
                    [
                        'service_id',
                        'host_id',
                        'sla_id',
                        'updated',
                    ]
                )
                ->addIndex(
                    [
                        'service_id',
                        'host_id',
                    ],
                    [
                        'unique' => true
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('sla_availability_status_hosts_log')) {
            $this->table('sla_availability_status_hosts_log')
                ->addColumn('id', 'biginteger', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id', 'evaluation_start']) // General error: 1503 A PRIMARY KEY must include all columns in the table's partitioning function (prefixed columns are not considered)
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('total_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Total time of the evaluation period 24h - 48h - 72h - 96h'
                ])
                ->addColumn('minimal_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Guaranteed available as a percentage'
                ])
                ->addColumn('determined_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Actual availability that was calculated by the SLA as percentage'
                ])
                ->addColumn('minimal_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Guaranteed available in seconds'
                ])
                ->addColumn('determined_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual availability that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_outage_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual outages that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_number_outages', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Number of outages'
                ])
                ->addColumn('up', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Up status in seconds'
                ])
                ->addColumn('down', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Down status in seconds'
                ])
                ->addColumn('unreachable', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Unreachable status in seconds'
                ])
                ->addColumn('evaluation_start', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Timestamp when the evaluation what started 01.03.2022 00:00:00'
                ])
                ->addColumn('evaluation_end', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Time when the evaluation what ended 01.03.2022 24:00:00'
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'host_id',
                        'sla_id'
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('sla_availability_status_services_log')) {
            $this->table('sla_availability_status_services_log')
                ->addColumn('id', 'biginteger', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id', 'evaluation_start']) //  General error: 1503 A PRIMARY KEY must include all columns in the table's partitioning function (prefixed columns are not considered)
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('service_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('total_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Total time of the evaluation period 24h - 48h - 72h - 96h'
                ])
                ->addColumn('minimal_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Guaranteed available as a percentage'
                ])
                ->addColumn('determined_availability_percent', 'float', [
                    'null'      => false,
                    'precision' => 8,
                    'scale'     => 3,
                    'comment'   => 'Actual availability that was calculated by the SLA as percentage'
                ])
                ->addColumn('minimal_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Guaranteed available in seconds'
                ])
                ->addColumn('determined_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual availability that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_outage_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Actual outages that was calculated by the SLA in seconds'
                ])
                ->addColumn('determined_number_outages', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Number of outages'
                ])
                ->addColumn('ok', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Ok status in seconds'
                ])
                ->addColumn('warning', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Warning status in seconds'
                ])
                ->addColumn('critical', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Critical status in seconds'
                ])
                ->addColumn('unknown', 'integer', [
                    'default' => 0,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Time with Unknown status in seconds'
                ])
                ->addColumn('evaluation_start', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Timestamp when the evaluation what started 01.03.2022 00:00:00'
                ])
                ->addColumn('evaluation_end', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                    'comment' => 'Time when the evaluation what ended 01.03.2022 24:00:00'
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'service_id',
                        'host_id',
                        'sla_id',
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('sla_host_outages')) {
            $this->table('sla_host_outages')
                ->addPrimaryKey(['host_id', 'start_time'])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'limit' => 11,
                    'null'  => false,
                ])
                ->addColumn('sla_availability_status_hosts_log_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('start_time', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('end_time', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('output', 'string', [
                    'default' => null,
                    'limit'   => 1024,
                    'null'    => true,
                ])
                ->addColumn('is_hardstate', 'boolean', [
                    'default' => false,
                    'null'    => false,
                ])
                ->addColumn('in_downtime', 'boolean', [
                    'default' => false,
                    'null'    => false,
                ])
                ->addIndex(['sla_id'])
                ->create();
        }

        if (!$this->hasTable('sla_service_outages')) {
            $this->table('sla_service_outages')
                ->addPrimaryKey(['service_id', 'start_time'])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('service_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('sla_id', 'integer', [
                    'limit' => 11,
                    'null'  => false,
                ])
                ->addColumn('sla_availability_status_services_log_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('start_time', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('end_time', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('output', 'string', [
                    'default' => null,
                    'limit'   => 1024,
                    'null'    => true,
                ])
                ->addColumn('is_hardstate', 'boolean', [
                    'default' => false,
                    'null'    => false,
                ])
                ->addColumn('in_downtime', 'boolean', [
                    'default' => false,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'service_id',
                        'host_id',
                    ]
                )
                ->addIndex(['sla_id'])
                ->create();
        }
    }

    /**
     * Down Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-down-method
     * @return void
     */
    public function down(): void {
        $this->table('sla_availability_status_hosts')->drop()->save();
        $this->table('sla_availability_status_services')->drop()->save();
        $this->table('sla_availability_status_log_hosts')->drop()->save();
        $this->table('sla_availability_status_log_services')->drop()->save();
        $this->table('sla_host_outages')->drop()->save();
        $this->table('sla_service_outages')->drop()->save();
    }
}
