<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class SlaSettings
 *
 * Created via:
 * oitc migrations create -p SLAModule SlaSettings
 *
 * Run migration:
 * oitc migrations migrate -p SLAModule
 *
 */
class SlaSettings extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {
        if (!$this->hasTable('sla_settings')) {
            $this->table('sla_settings')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('max_age', 'integer', [
                    'default' => 53,
                    'limit'   => 11,
                    'null'    => false,
                    'comment' => 'Max age in weeks of database partitions'
                ])
                ->create();
        }

    }
}
