<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace SLAModule\Controller;

use App\Model\Entity\Host;
use App\Model\Table\HostsTable;
use Cake\Datasource\ResultSetInterface;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\AngularRequest;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use SLAModule\Lib\SlaStatusLogConditions;
use SLAModule\Model\Entity\SlaAvailabilityStatusHostLog;
use SLAModule\Model\Table\SlaAvailabilityStatusHostsLogTable;


/**
 * SlaAvailabilityStatusHostsLog Controller
 *
 * @method SlaAvailabilityStatusHostLog[]|ResultSetInterface paginate($object = null, array $settings = [])
 */
class SlaAvailabilityStatusHostsLogController extends AppController {

    public function index($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        $session = $this->request->getSession();
        $session->close();

        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if (!$HostsTable->existsById($id)) {
            throw new NotFoundException(__('Invalid host'));
        }

        /** @var Host $host */
        $host = $HostsTable->getHostByIdForPermissionCheck($id);
        if (!$this->allowedByContainerId($host->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $AngularRequest = new AngularRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AngularRequest->getPage());


        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        //Process conditions
        $Conditions = new SlaStatusLogConditions();
        $Conditions->setOrder($AngularRequest->getOrderForPaginator('SlaAvailabilityStatusHosts.evaluation_start', 'desc'));
        $Conditions->setFrom($AngularRequest->getFrom());
        $Conditions->setTo($AngularRequest->getTo());
        $Conditions->setId($host->get('id'));

        /** @var SlaAvailabilityStatusHostsLogTable $SlaAvailabilityStatusHostsLogTable */
        $SlaAvailabilityStatusHostsLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusHostsLog');


        $slaHostStatusLog = $SlaAvailabilityStatusHostsLogTable->getSlaHostStatusLogIndex($Conditions, $PaginateOMat, false);
        //Parse BBCode in output
        $BBCodeParser = new BBCodeParser();
        foreach ($slaHostStatusLog as $key => $slaHostStatus) {
            $slaHostStatusLog[$key]['evaluation_total_time_human_readable'] = $UserTime->secondsInHumanShort($slaHostStatus['evaluation_total_time']);
            $slaHostStatusLog[$key]['determined_availability_time_human_readable'] = $UserTime->secondsInHumanShort($slaHostStatus['determined_availability_time']);
            $slaHostStatusLog[$key]['up_human_readable'] = $UserTime->secondsInHumanShort($slaHostStatus['up']);
            $slaHostStatusLog[$key]['down_human_readable'] = $UserTime->secondsInHumanShort($slaHostStatus['down']);
            $slaHostStatusLog[$key]['unreachable_human_readable'] = $UserTime->secondsInHumanShort($slaHostStatus['unreachable']);
            $slaHostStatusLog[$key]['start'] = $UserTime->format($slaHostStatus['evaluation_start']);
            $slaHostStatusLog[$key]['end'] = $UserTime->format($slaHostStatus['evaluation_end']);
            $slaHostStatusLog[$key]['state'] = 'success';
            if ($slaHostStatus['determined_availability_percent'] < $slaHostStatus['minimal_availability_percent']) {
                $slaHostStatusLog[$key]['state'] = 'danger';
            }
            $outageTime = 0;
            foreach ($slaHostStatus['sla_host_outages'] as $outageKey => $outage) {
                $slaHostStatusLog[$key]['sla_host_outages'][$outageKey]['outputHtml'] = $BBCodeParser->nagiosNl2br(
                    $BBCodeParser->asHtml($outage['output'], true)
                );
                $slaHostStatusLog[$key]['sla_host_outages'][$outageKey]['duration'] = $UserTime->secondsInHumanShort(
                    $outage['end_time'] - $outage['start_time']
                );
                if ($outage['in_downtime'] == 0) {
                    $outageTime += $outage['end_time'] - $outage['start_time'];
                }

                $slaHostStatusLog[$key]['sla_host_outages'][$outageKey]['start'] = $UserTime->format($outage['start_time']);
                $slaHostStatusLog[$key]['sla_host_outages'][$outageKey]['end'] = $UserTime->format($outage['end_time']);
            }
            $slaHostStatusLog[$key]['determined_outage_time_human_readable'] = $UserTime->secondsInHumanShort($outageTime);

        }

        $this->set('host', $host);
        $this->set('slaHostStatusLog', $slaHostStatusLog);

        $this->viewBuilder()->setOption('serialize', ['host', 'slaHostStatusLog']);
    }
}
