<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace SLAModule\Controller;

use App\Model\Entity\Service;
use App\Model\Table\ServicesTable;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\AngularJS\Request\AngularRequest;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\BBCodeParser;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use SLAModule\Lib\SlaStatusLogConditions;
use SLAModule\Model\Entity\SlaAvailabilityStatusServiceLog;
use SLAModule\Model\Table\SlaAvailabilityStatusServicesLogTable;

class SlaAvailabilityStatusServicesLogController extends AppController {

    public function index($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        $session = $this->request->getSession();
        $session->close();

        /** @var $ServicesTable ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        if (!$ServicesTable->existsById($id)) {
            throw new NotFoundException(__('Invalid service'));
        }

        /** @var Service $service */
        $service = $ServicesTable->getServiceByIdForPermissionsCheck($id);
        if (!$this->allowedByContainerId($service->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $AngularRequest = new AngularRequest($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AngularRequest->getPage());


        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        //Process conditions
        $Conditions = new SlaStatusLogConditions();
        $Conditions->setOrder($AngularRequest->getOrderForPaginator('SlaAvailabilityStatusServices.evaluation_start', 'desc'));
        $Conditions->setFrom($AngularRequest->getFrom());
        $Conditions->setTo($AngularRequest->getTo());
        $Conditions->setId($service->get('id'));

        /** @var SlaAvailabilityStatusServicesLogTable $SlaAvailabilityStatusServicesLogTable */
        $SlaAvailabilityStatusServicesLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusServicesLog');


        $slaServiceStatusLog = $SlaAvailabilityStatusServicesLogTable->getSlaServiceStatusLogIndex($Conditions, $PaginateOMat, false);
        //Parse BBCode in output
        $BBCodeParser = new BBCodeParser();
        foreach ($slaServiceStatusLog as $key => $slaServiceStatus) {
            $slaServiceStatusLog[$key]['evaluation_total_time_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['evaluation_total_time']);
            $slaServiceStatusLog[$key]['determined_availability_time_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['determined_availability_time']);
            $slaServiceStatusLog[$key]['ok_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['ok']);
            $slaServiceStatusLog[$key]['warning_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['warning']);
            $slaServiceStatusLog[$key]['critical_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['critical']);
            $slaServiceStatusLog[$key]['unknown_human_readable'] = $UserTime->secondsInHumanShort($slaServiceStatus['unknown']);
            $slaServiceStatusLog[$key]['start'] = $UserTime->format($slaServiceStatus['evaluation_start']);
            $slaServiceStatusLog[$key]['end'] = $UserTime->format($slaServiceStatus['evaluation_end']);
            $slaServiceStatusLog[$key]['state'] = 'success';
            if ($slaServiceStatus['determined_availability_percent'] < $slaServiceStatus['minimal_availability_percent']) {
                $slaServiceStatusLog[$key]['state'] = 'danger';
            }
            $outageTime = 0;
            foreach ($slaServiceStatus['sla_service_outages'] as $outageKey => $outage) {
                $slaServiceStatusLog[$key]['sla_service_outages'][$outageKey]['outputHtml'] = $BBCodeParser->nagiosNl2br(
                    $BBCodeParser->asHtml($outage['output'], true)
                );
                $slaServiceStatusLog[$key]['sla_service_outages'][$outageKey]['duration'] = $UserTime->secondsInHumanShort(
                    $outage['end_time'] - $outage['start_time']
                );
                if ($outage['in_downtime'] == 0) {
                    $outageTime += $outage['end_time'] - $outage['start_time'];
                }

                $slaServiceStatusLog[$key]['sla_service_outages'][$outageKey]['start'] = $UserTime->format($outage['start_time']);
                $slaServiceStatusLog[$key]['sla_service_outages'][$outageKey]['end'] = $UserTime->format($outage['end_time']);
            }
            $slaServiceStatusLog[$key]['determined_outage_time_human_readable'] = $UserTime->secondsInHumanShort($outageTime);

        }

        $this->set('service', $service);
        $this->set('slaServiceStatusLog', $slaServiceStatusLog);

        $this->viewBuilder()->setOption('serialize', ['service', 'slaServiceStatusLog']);
    }
}
