<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace SLAModule\Form;

use Cake\Form\Form;
use Cake\Form\Schema;
use Cake\Validation\Validator;

/**
 * Class SlaReportForm
 * @package SLAModule\Form
 */
class SlaReportForm extends Form {
    protected function _buildSchema(Schema $schema): Schema {
        return $schema
            ->addField('id', ['type' => 'string'])
            ->addField('from_date', ['type' => 'string'])
            ->addField('to_date', ['type' => 'string']);
    }

    public function validationDefault(Validator $validator): Validator {
        $validator
            ->requirePresence('id')
            ->allowEmptyString('id', null, false);

        $validator
            ->date('from_date', ['dmy'])
            ->requirePresence('from_date')
            ->allowEmptyDateTime('from_date', null, false);

        $validator
            ->date('to_date', ['dmy'])
            ->requirePresence('to_date')
            ->allowEmptyDateTime('to_date', null, false)
            ->add('to_date', 'custom', [
                'rule'    => function ($value, $context) {
                    $fromDate = strtotime($context['data']['from_date'] . ' 00:00:00');
                    $toDate = strtotime($value . ' 23:59:59');
                    return !($fromDate > $toDate);
                },
                'message' => '"From" date should be greater then "to" date.'
            ]);
        return $validator;
    }
}
