<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace SLAModule\Lib;


use itnovum\openITCOCKPIT\Core\Reports\CsvCreator;

/**
 * Class CsvReport
 * @package SLAModule\Lib
 */
class CsvReport {

    /**
     * @var array
     */
    private $report;


    /** @var string */
    private $path;


    /**
     * CsvReport constructor.
     * @param $this ->report
     * @param $path
     */
    public function __construct($report, $path) {
        $this->report = $report;
        $this->path = $path;
    }

    public function getReportAsZip() {
        $filename = $this->report['sla']['name'];
        $filenameSuffix = "_" . date("Y-m-d_His");

        $filename = $filename . $filenameSuffix . '.zip';
        $zipName = $this->path . $filename;
        $AvailabilityCsvFiles = [];
        $OutagesCsvFiles = [];
        $Csv = new CsvCreator();

        foreach ($this->report['date_ranges'] as $key => $dateRange) {
            $dateRangeTitle = preg_replace('/\s+/', '', $dateRange['title']);
            $csvDate = sprintf(
                '%s_%s_%s',
                $this->report['sla']['name'],
                date('Y-m-d_H-i-s', $dateRange['start']),
                date('Y-m-d_H-i-s', $dateRange['end'])
            );
            $AvailabilityCsv = $this->path . $csvDate . '_Availability_' . $dateRangeTitle . '.csv';

            $outages = [];
            if (!empty($this->report['hosts'][$key])) {

                $Csv->setPath($AvailabilityCsv);
                $Csv->addRow([
                    __('Location'),
                    __('System'),
                    __('Service'),
                    __('UP_%'),
                    __('DOWN_%'),
                    __('UP_Seconds'),
                    __('DOWN_Seconds')
                ]);


                foreach ($this->report['hosts'][$key] as $slaHost) {
                    if (!empty($slaHost['sla_log']['last_log_entry'])) {
                        $slaLogLastEntry = $slaHost['sla_log']['last_log_entry'];
                        $Csv->addRow([
                            $slaHost['host']['primary_container'],
                            $slaHost['host']['name'],
                            null,
                            $slaLogLastEntry['determined_availability_percent'],
                            floatval(sprintf('%0.3f', 100 - $slaLogLastEntry['determined_availability_percent'])),
                            $slaLogLastEntry['determined_availability_time'],
                            $slaLogLastEntry['determined_outage_time']
                        ]);
                    }
                    if (!empty($slaHost['sla_log']['outages'])) {
                        foreach ($slaHost['sla_log']['outages'] as $outage) {
                            $outages[$key][] = [
                                $slaHost['host']['name'],
                                null,
                                date('d.m.Y', $outage['start_time']),
                                date('H:i:s', $outage['start_time']),
                                date('d.m.Y', $outage['end_time']),
                                date('H:i:s', $outage['end_time']),
                                $outage['end_time'] - $outage['start_time']
                            ];
                        }
                    }
                    if (!empty($slaHost['services'])) {
                        foreach ($slaHost['services'] as $service) {
                            if (!empty($service['sla_log']['last_log_entry'])) {
                                $slaLogLastEntry = $service['sla_log']['last_log_entry'];
                                $Csv->addRow([
                                    $slaHost['host']['primary_container'],
                                    $slaHost['host']['name'],
                                    $service['service']['servicename'],
                                    $slaLogLastEntry['determined_availability_percent'],
                                    floatval(sprintf('%0.3f', 100 - $slaLogLastEntry['determined_availability_percent'])),
                                    $slaLogLastEntry['determined_availability_time'],
                                    $slaLogLastEntry['determined_outage_time']
                                ]);
                            }
                            if (!empty($service['sla_log']['outages'])) {
                                foreach ($service['sla_log']['outages'] as $outage) {
                                    $outages[$key][] = [
                                        $slaHost['host']['name'],
                                        $service['service']['servicename'],
                                        date('d.m.Y', $outage['start_time']),
                                        date('H:i:s', $outage['start_time']),
                                        date('d.m.Y', $outage['end_time']),
                                        date('H:i:s', $outage['end_time']),
                                        $outage['end_time'] - $outage['start_time']
                                    ];
                                }
                            }

                        }
                    }
                }

                $Csv->closeFile();
                $AvailabilityCsvFiles[] = $AvailabilityCsv;

                $OutagesCSV = $this->path . $csvDate . '_Outages_' . $dateRangeTitle . '.csv';
                $Csv->setPath($OutagesCSV);
                $Csv->addRow([
                    __('System'),
                    __('Service'),
                    __('Start Date'),
                    __('Start Time'),
                    __('End Date'),
                    __('End Time'),
                    __('Down Period')
                ]);
                if (!empty($outages[$key])) {
                    foreach ($outages[$key] as $outageRow) {
                        $Csv->addRow($outageRow);
                    }
                }
                $Csv->closeFile();
                $AvailabilityCsvFiles[] = $OutagesCSV;
            }
        }
        $Csv->createZip($zipName, array_merge($AvailabilityCsvFiles, $OutagesCsvFiles));
        return $zipName;
    }
}
