<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace SLAModule\Lib;


use App\Model\Table\ServicesTable;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use SLAModule\Model\Table\SlaAvailabilityStatusHostsLogTable;
use SLAModule\Model\Table\SlaAvailabilityStatusServicesLogTable;
use SLAModule\Model\Table\SlasTable;
use SLAModule\Reports\SlaDaterangesCreator;

class SlaReport {


    /**
     * @var array
     */
    private $sla;

    /**
     * @var int
     */
    private $startTimestamp;

    /**
     * @var int
     */
    private $endTimestamp;

    /**
     * @var int
     */
    private $evaluation;

    /**
     * @var UserTime
     */
    private $UserTime;

    /**
     * @var array
     */
    private $hosts;

    public function __construct(int $id, array $hosts, int $startTimestamp, int $endTimestamp, UserTime $UserTime, int $evaluation) {
        /** @var SlasTable $SlasTable */
        $SlasTable = TableRegistry::getTableLocator()->get('SLAModule.Slas');

        if (!$SlasTable->existsById($id)) {
            throw new NotFoundException();
        }
        $this->sla = $SlasTable->getSlaById($id);
        $this->hosts = $hosts;

        $this->evaluation = $evaluation;
        $this->startTimestamp = $startTimestamp;
        $this->endTimestamp = $endTimestamp;

        /*
        if (!empty($this->sla->get('start_date'))) {
            $tmpStartTimestamp = strtotime($this->sla->get('start_date') . ' 00:00:00');
            if ($tmpStartTimestamp > $this->startTimestamp) {
                $this->startTimestamp = $tmpStartTimestamp;
            }
        }
        if ($this->endTimestamp > time()) {
            $this->endTimestamp = time();
        }
*/
        $this->UserTime = $UserTime;

        $weekdays = [
            1 => __('Monday'),
            2 => __('Tuesday'),
            3 => __('Wednesday'),
            4 => __('Thursday'),
            5 => __('Friday'),
            6 => __('Saturday'),
            7 => __('Sunday'),
        ];
        foreach ($this->sla['timeperiod']['timeperiod_timeranges'] as $index => $timerange) {
            $this->sla['timeperiod']['timeperiod_timeranges'][$index]['weekday'] = $weekdays[$timerange['day']];
        }
    }


    /**
     * @param bool $prettyFormat
     * @param bool $formatOutages
     * @return array
     * @throws \Exception
     */
    public function generateReport(bool $prettyFormat = true, bool $formatOutages = false) {
        $sla_data['sla'] = $this->sla->toArray();
        if (!empty($sla_data['sla']['start_date'])) {
            $date = \DateTime::createFromFormat('Y-m-d', $sla_data['sla']['start_date']);
            $sla_data['sla']['start_date'] = $date->format('d.m.Y');
        }
        //$sla_data['sla']['start_date']
        if ($prettyFormat === true) {
            $sla_data['evaluation_time'] = [
                'start'       => $this->startTimestamp,
                'start_human' => $this->UserTime->format($this->startTimestamp),
                'end'         => $this->endTimestamp,
                'end_human'   => $this->UserTime->format($this->endTimestamp)
            ];

            $sla_data['timeperiod'] = $this->sla['timeperiod']->toArray();
        }


        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            $this->sla->get('evaluation_interval'),
            $this->startTimestamp,
            $this->endTimestamp
        );
        foreach ($dateRanges as $key => $dateRange) {
            $dateRanges[$key]['start_human'] = $this->UserTime->format($dateRange['start']);
            $dateRanges[$key]['end_human'] = $this->UserTime->format($dateRange['end']);
        }

        $sla_data['date_ranges'] = $dateRanges;

        /** @var SlaAvailabilityStatusHostsLogTable $SlaAvailabilityStatusHostsLogTable */
        $SlaAvailabilityStatusHostsLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusHostsLog');

        /** @var SlaAvailabilityStatusServicesLogTable $SlaAvailabilityStatusServicesLogTable */
        $SlaAvailabilityStatusServicesLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusServicesLog');

        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        foreach ($this->hosts as $host) {
            if ($this->evaluation !== 3) {
                foreach ($dateRanges as $key => $dateRange) {
                    $slaLogForHost = $SlaAvailabilityStatusHostsLogTable->getSlaHostStatusLogByHostIdWithRange(
                        $host['id'],
                        $dateRange['start'],
                        $dateRange['end']
                    );

                    $outages = [];
                    $outagesInDowntime = [];

                    $slaLog = [];
                    $numberOfRecord = sizeof($slaLogForHost);

                    foreach ($slaLogForHost as $logForHostKey => $slaRecord) {
                        foreach ($slaRecord['sla_host_outages'] as $outage) {
                            if ($formatOutages === true) {
                                $outage['start_human_readable'] = $this->UserTime->format($outage['start_time']);
                                $outage['end_human_readable'] = $this->UserTime->format($outage['end_time']);
                                $outage['total_time'] = $this->UserTime->secondsInHumanShort(
                                    $outage['end_time'] - $outage['start_time']
                                );
                            }
                            if ($outage['in_downtime'] === true) {
                                $outagesInDowntime[] = $outage;

                            } else {
                                $outages[] = $outage;
                            }
                        }
                        if ($logForHostKey === ($numberOfRecord - 1)) { // <--- detect latest entry for log for selected range
                            $slaLog = $slaRecord;
                            $slaLog['evaluation_total_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                $slaRecord['evaluation_total_time']
                            );
                            $slaLog['determined_availability_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                $slaRecord['determined_availability_time']
                            );
                            $slaLog['determined_outage_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                $slaRecord['determined_outage_time']
                            );
                            $slaLog['passed'] = $slaRecord['determined_availability_percent'] > $slaRecord['minimal_availability_percent'];
                        }
                    }

                    $sla_data['hosts'][$key][$host['id']] = [
                        'host'    => $host,
                        'sla_log' => [
                            'last_log_entry'      => $slaLog,
                            'outages'             => $outages,
                            'outages_in_downtime' => $outagesInDowntime
                        ]
                    ];
                }
            }
            if ($this->evaluation !== 1) {
                $services = $ServicesTable->getActiveServicesWithServicetemplateByHostId($host['id'], false);
                foreach ($services as $service) {
                    foreach ($dateRanges as $key => $dateRange) {
                        $slaLogForService = $SlaAvailabilityStatusServicesLogTable->getSlaServiceStatusLogByServiceIdWithRange(
                            $service['id'],
                            $dateRange['start'],
                            $dateRange['end']
                        );
                        $outages = [];
                        $outagesInDowntime = [];
                        $slaLog = [];
                        $numberOfRecord = sizeof($slaLogForService);
                        foreach ($slaLogForService as $slaLogForServiceKey => $slaRecord) {
                            foreach ($slaRecord['sla_service_outages'] as $outage) {
                                if ($formatOutages === true) {
                                    $outage['start_human_readable'] = $this->UserTime->format($outage['start_time']);
                                    $outage['end_human_readable'] = $this->UserTime->format($outage['end_time']);
                                    $outage['total_time'] = $this->UserTime->secondsInHumanShort(
                                        $outage['end_time'] - $outage['start_time']
                                    );
                                }
                                if ($outage['in_downtime'] === true) {
                                    $outagesInDowntime[] = $outage;
                                } else {
                                    $outages[] = $outage;
                                }
                            }
                            if ($slaLogForServiceKey === ($numberOfRecord - 1)) { // <--- detect latest entry for log for selected range
                                $slaLog = $slaRecord;
                                $slaLog['evaluation_total_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                    $slaRecord['evaluation_total_time']
                                );
                                $slaLog['determined_availability_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                    $slaRecord['determined_availability_time']
                                );
                                $slaLog['determined_outage_time_human_readable'] = $this->UserTime->secondsInHumanShort(
                                    $slaRecord['determined_outage_time']
                                );
                                $slaLog['passed'] = $slaRecord['determined_availability_percent'] > $slaRecord['minimal_availability_percent'];
                            }
                        }
                        if (!isset($sla_data['hosts'][$key][$host['id']]['host'])) {
                            $sla_data['hosts'][$key][$host['id']]['host'] = $host;
                        }
                        $sla_data['hosts'][$key][$host['id']]['services'][] = [
                            'service' => $service,
                            'sla_log' => [
                                'last_log_entry'      => $slaLog,
                                'outages'             => $outages,
                                'outages_in_downtime' => $outagesInDowntime
                            ]
                        ];
                    }
                }
            }
        }

        if (!empty($sla_data['hosts'])) {
            foreach ($sla_data['hosts'] as $key => $hostLog) {
                //keep alphabetical sorting
                $sla_data['hosts'][$key] = array_values($sla_data['hosts'][$key]);
            }
        }

        return $sla_data;
    }

    public function canSlaBeCalculated() {
        if (!empty($this->sla['start_date'])) {
            $startDateAsTimestamp = strtotime($this->sla['start_date'] . ' 00:00:00');

            if ($this->startTimestamp < $startDateAsTimestamp && $this->endTimestamp < $startDateAsTimestamp) {
                return [
                    'error'   => true,
                    'message' => __(
                        'SLA "{0}" cannot be evaluated. The "from" and "to" date of the evaluation is before sla start date "{1}"',
                        $this->sla['name'],
                        $this->UserTime->format(strtotime($this->sla['start_date'] . ' 00:00:00'))
                    ),
                    'objects' => []
                ];
            }
            if ($this->startTimestamp < $startDateAsTimestamp) {
                return [
                    'error'   => true,
                    'message' => __(
                        'SLA "{0}" cannot be evaluated. The "from" date of the evaluation is before sla start date "{1}"',
                        $this->sla['name'],
                        $this->UserTime->format(strtotime($this->sla['start_date'] . ' 00:00:00'))
                    ),
                    'objects' => []
                ];
            }
        }

        return true;
    }
}
