<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace SLAModule\Lib;


use App\Lib\Interfaces\ServicestatusTableInterface;
use App\Model\Entity\Service;
use App\Model\Table\SystemfailuresTable;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\DowntimeServiceConditions;
use itnovum\openITCOCKPIT\Core\Reports\DaterangesCreator;
use itnovum\openITCOCKPIT\Core\Reports\StatehistoryConverter;
use itnovum\openITCOCKPIT\Core\Servicestatus;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\StatehistoryServiceConditions;
use itnovum\openITCOCKPIT\Core\Views\Downtime;
use itnovum\openITCOCKPIT\Core\Views\StatehistoryService;
use SLAModule\Model\Table\SlaAvailabilityStatusServicesLogTable;
use SLAModule\Model\Table\SlaAvailabilityStatusServicesTable;
use SLAModule\Reports\SlaDaterangesCreator;

class SlaService {
    /**
     * @var \SLAModule\Model\Entity\Sla $sla
     */
    private $Sla;

    /**
     * @var Service
     */
    private $Service;

    /**
     * @var DbBackend
     */
    private $DbBackend;

    /**
     * @param \SLAModule\Model\Entity\Sla $Sla
     * @param Service $service
     */
    public function __construct(\SLAModule\Model\Entity\Sla $Sla, Service $service) {
        $this->Sla = $Sla->toArray();
        $this->Service = $service;
        $this->DbBackend = new DbBackend();
    }

    /**
     * @return SlaResult
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     */
    public function calculateAvailabilityForService(): SlaResult {
        /** @var SlaAvailabilityStatusServicesTable $SlaAvailabilityStatusServicesTable */
        $SlaAvailabilityStatusServicesTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusServices');

        $firstRun = true;
        $slaStartDate = DaterangesCreator::getInitStartTimeForCalculateAvailability($this->Sla['evaluation_interval']);

        //use start date of sla if defined
        if (!empty($this->Sla['start_date'])) {
            $slaStartDate = $this->Sla['start_date']->timestamp;
        }

        if ($slaStartDate < $this->Service->created->getTimestamp()) {
            // Host was created after SLA start date
            $slaStartDate = $this->Service->created->getTimestamp();
        }

        if (!empty($this->Service['sla_availability_status_services_log'])) {
            $lastRecordFromLog = $this->Service['sla_availability_status_services_log'][0];
            $slaStartDate = (int)$lastRecordFromLog['evaluation_end']->toUnixString();
            $firstRun = false;
        }

        //use date of last calculated sla if exists
        if (!empty($this->Service['sla_availability_status_service'])) {
            $lastRecordFromLog = $this->Service['sla_availability_status_service'];
            $slaStartDate = (int)$lastRecordFromLog['evaluation_end']->toUnixString();
            $firstRun = false;
        } else {
            // Check the SlaAvailabilityStatusServicesLog Table if records exists
            /** @var SlaAvailabilityStatusServicesLogTable $SlaAvailabilityStatusServicesLogTable */
            $SlaAvailabilityStatusServicesLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusServicesLog');

            $query = $SlaAvailabilityStatusServicesLogTable->find()
                ->select([
                    'SlaAvailabilityStatusServicesLog.service_id',
                    'SlaAvailabilityStatusServicesLog.evaluation_end',
                    'SlaAvailabilityStatusServicesLog.evaluation_total_time',
                    'SlaAvailabilityStatusServicesLog.total_time',
                    'SlaAvailabilityStatusServicesLog.determined_outage_time',
                    'SlaAvailabilityStatusServicesLog.determined_availability_percent',
                    'SlaAvailabilityStatusServicesLog.determined_number_outages'

                ])
                ->where([
                    'SlaAvailabilityStatusServicesLog.service_id' => $this->Service->get('id')
                ])
                ->order(['SlaAvailabilityStatusServicesLog.id' => 'desc'])
                ->disableHydration()
                ->limit(1);
            $lastRecordFromLog = $query->first();
            if (!empty($lastRecordFromLog)) {
                $slaStartDate = (int)$lastRecordFromLog['evaluation_end']->toUnixString();
                $firstRun = false;
            }
        }

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $slaStartDate,
            $this->Sla['evaluation_interval'],
            $firstRun
        );

        /** @var SlaAvailabilityStatusServicesLogTable $SlaAvailabilityStatusServicesLogTable */
        $SlaAvailabilityStatusServicesLogTable = TableRegistry::getTableLocator()->get('SLAModule.SlaAvailabilityStatusServicesLog');

        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();
        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields
            ->currentState()
            ->isHardstate()
            ->lastStateChange()
            ->lastHardState()
            ->lastHardStateChange()
            ->maxCheckAttempts()
            ->currentCheckAttempt()
            ->output();
        $servicestatus = $ServicestatusTable->byUuid($this->Service->get('uuid'), $ServicestatusFields);
        $statehistoryRecords = [];
        if (!empty($servicestatus)) {
            $Servicestatus = new Servicestatus($servicestatus['Servicestatus']);
            $lastStateChange = max($Servicestatus->getLastStateChange(), $Servicestatus->getLastHardStateChange());
            if ($this->Sla['hard_state_only']) {
                $lastStateChange = $Servicestatus->getLastHardStateChange();
            }

            if ($lastStateChange > $slaStartDate) {
                //state change since last sla calculation
                $StatehistoryServicesTable = $this->DbBackend->getStatehistoryServicesTable();
                $Conditions = new StatehistoryServiceConditions();
                $Conditions->setFrom($slaStartDate);
                $Conditions->setServiceUuid($this->Service->get('uuid'));
                $Conditions->setHardStateTypeAndOkState($this->Sla['hard_state_only']);

                $statehistoryRecords = $StatehistoryServicesTable->getRecordsForReporting($Conditions);
            } else {
                // no state change since last sla calculation
                $statehistoryRecords = [
                    new StatehistoryService([
                        'hostname'              => $this->Service->get('uuid'),
                        'state_time'            => $lastStateChange,
                        'state_time_usec'       => 0,
                        'state_change'          => false,
                        'state'                 => $Servicestatus->currentState(),
                        'is_hardstate'          => $Servicestatus->isHardState(),
                        'current_check_attempt' => $Servicestatus->getCurrentCheckAttempt(),
                        'max_check_attempts'    => $Servicestatus->getMaxCheckAttempts(),
                        'last_state'            => $Servicestatus->currentState(),
                        'last_hard_state'       => $Servicestatus->getLastHardState(),
                        'output'                => $Servicestatus->getOutput(),
                        'long_output'           => null
                    ])
                ];
            }
        }
        $downtimes = [];
        if ($this->Sla['consider_downtimes']) {
            /** @var SystemfailuresTable $SystemfailuresTable */
            $SystemfailuresTable = TableRegistry::getTableLocator()->get('Systemfailures');

            $systemFailures = $SystemfailuresTable->getSystemfailuresForReportingFormatedAsDowntime(
                $slaStartDate,
                time()
            );

            $DowntimeServiceConditions = new DowntimeServiceConditions();
            $DowntimeServiceConditions->setOrder(['DowntimeServices.scheduled_start_time' => 'asc']);
            $DowntimeServiceConditions->setFrom($slaStartDate);
            $DowntimeServiceConditions->setTo(time());
            $DowntimeServiceConditions->setServiceUuid($this->Service->get('uuid'));

            $DowntimehistoryServicesTable = $this->DbBackend->getDowntimehistoryServicesTable();
            $downtimes = $DowntimehistoryServicesTable->getDowntimesForReporting($DowntimeServiceConditions, false, true);
            $formatedDowntimes = [];
            foreach ($downtimes as $downtime) {
                //Use the same format for downtimes and systemdowntimes (also with Statusengine 2 and Statusengine 3)
                $Downtime = new Downtime($downtime);
                $endTime = $Downtime->getScheduledEndTime();
                if ($Downtime->wasCancelled()) {
                    $endTime = $Downtime->getActualEndTime();
                }

                $formatedDowntimes[] = [
                    'author_name'          => $Downtime->getAuthorName(),
                    'comment_data'         => $Downtime->getCommentData(),
                    'entry_time'           => date('Y-m-d H:i:s', $Downtime->getEntryTime()),
                    'scheduled_start_time' => date('Y-m-d H:i:s', $Downtime->getScheduledStartTime()),
                    'scheduled_end_time'   => date('Y-m-d H:i:s', $endTime),
                    'duration'             => $Downtime->getDuration(),
                    'was_started'          => (int)$Downtime->wasStarted(),
                    'internal_downtime_id' => $Downtime->getInternalDowntimeId(),
                    'was_cancelled'        => (int)$Downtime->wasCancelled()
                ];
            }

            $downtimes = Hash::sort(
                Hash::filter(
                    array_merge(
                        $systemFailures['Systemfailure'],
                        $formatedDowntimes
                    )
                ), '{n}.scheduled_start_time', 'ASC'
            );
            if (!empty($downtimes)) {
                $downtimes = DaterangesCreator::mergeTimeOverlapping(
                    array_map(
                        function ($downtimes) {
                            return [
                                'start_time' => strtotime($downtimes['scheduled_start_time']),
                                'end_time'   => strtotime($downtimes['scheduled_end_time']),
                            ];
                        },
                        $downtimes
                    )
                );
            }
        }

        foreach ($dateRanges as $dateRange) {
            $currentStart = new \DateTime(date('d.m.Y H:i:s', $dateRange['start']));
            $i = 0;
            $totalTime = 0;
            $totalOutageTime = 0;
            $availabilityInPercent = 100;
            $totalNumberOfOutages = 0;
            $evaluationTotalTime = $dateRange['end'] - $dateRange['start']; //due to the time change, March has one hour less
            if ($dateRange['entryType'] === 'new') {
                $allTimeSlicesInEvalutionPeriod = SlaDaterangesCreator::createDateRangesForSla(
                    $dateRange['start'],
                    $dateRange['end'],
                    $this->Sla['timeperiod']['timeperiod_timeranges'],
                    $this->Sla['timeperiod']['calendar']
                );

                $evaluationTotalTime = Hash::apply(
                    Hash::map(
                        $allTimeSlicesInEvalutionPeriod, '{n}', [DaterangesCreator::class, 'calculateTotalTime']),
                    '{n}',
                    'array_sum'
                );
            }
            if ($dateRange['entryType'] === 'update' && !empty($lastRecordFromLog)) {
                $evaluationTotalTime = $lastRecordFromLog['evaluation_total_time'];
                $totalTime = $lastRecordFromLog['total_time'];
                $totalOutageTime = $lastRecordFromLog['determined_outage_time'];
                $availabilityInPercent = $lastRecordFromLog['determined_availability_percent'];
                $totalNumberOfOutages = $lastRecordFromLog['determined_number_outages'];
            }
            while ($currentStart->getTimestamp() < $dateRange['end'] && ($currentStart->getTimestamp() + 24 * 3600) < time()) {
                try {
                    $timeSlicesForCurrentDay = $this->getTimeSlices(
                        $currentStart->getTimestamp(),
                        $currentStart->getTimestamp() + 24 * 3600
                    );

                    $timeSlicesForCurrentDay = DaterangesCreator::setDowntimesInTimeslices(
                        $timeSlicesForCurrentDay,
                        $downtimes
                    );
                    $statehistoryRecordsArray = [];
                    foreach ($statehistoryRecords as $statehistoryRecord) {
                        $statehistoryRecordsArray[] = $statehistoryRecord->toArray();
                    }
                    $data = StatehistoryConverter::generateReportDataWithOutages(
                        $timeSlicesForCurrentDay,
                        $statehistoryRecordsArray,
                        $this->Sla['hard_state_only'],
                        false,
                        true
                    );

                    $totalTime += Hash::apply(
                        Hash::map(
                            $timeSlicesForCurrentDay, '{n}', [DaterangesCreator::class, 'calculateTotalTime']),
                        '{n}',
                        'array_sum'
                    );
                    $totalOutageTime += Hash::apply(Hash::map($data['outages'], '{n}', [DaterangesCreator::class, 'calculateTotalTime']), '{n}', 'array_sum');
                    $totalNumberOfOutages += sizeof($data['outages']);
                    $availabilityInPercent = strval(round(100 * ($evaluationTotalTime - $totalOutageTime) / $evaluationTotalTime, 3));

                    $logOutages = [];
                    $data['outages'] = array_merge($data['outages'], $data['outages_summarized_in_downtime']);
                    foreach ($data['outages'] as $outage) {
                        $logOutages[] = [
                            'host_id'      => $this->Service->host_id,
                            'service_id'   => $this->Service->id,
                            'sla_id'       => $this->Sla['id'],
                            'start_time'   => $outage['start'],
                            'end_time'     => $outage['end'],
                            'output'       => $outage['output'],
                            'is_hardstate' => $outage['is_hardstate'],
                            'in_downtime'  => $outage['is_downtime'] ?? null
                        ];
                    }
                    $slaAvailabilityStatusServicesLog[] = $SlaAvailabilityStatusServicesLogTable->newEntity([
                            'host_id'                         => $this->Service->host_id,
                            'service_id'                      => $this->Service->id,
                            'sla_id'                          => $this->Sla['id'],
                            'evaluation_total_time'           => $evaluationTotalTime,
                            'total_time'                      => $totalTime,
                            'evaluation_start'                => date('Y-m-d H:i:s', $currentStart->getTimestamp()),
                            'evaluation_end'                  => date('Y-m-d H:i:s', $currentStart->getTimestamp() + 24 * 3600),
                            'minimal_availability_time'       => (int)($totalTime * $this->Sla['minimal_availability'] / 100),
                            'determined_availability_time'    => $totalTime - $totalOutageTime,
                            'minimal_availability_percent'    => $this->Sla['minimal_availability'],
                            'determined_availability_percent' => $availabilityInPercent,
                            'determined_outage_time'          => $totalOutageTime,
                            'determined_number_outages'       => $totalNumberOfOutages,
                            'ok'                              => $data[0],
                            'warning'                         => $data[1],
                            'critical'                        => $data[2],
                            'unknown'                         => $data[3],
                            'sla_service_outages'             => $logOutages
                        ]
                    );

                } catch (\Exception $e) {
                    //if time slices are empty for example on 8x5 time range
                    $slaAvailabilityStatusServicesLog[] = $SlaAvailabilityStatusServicesLogTable->newEntity([
                        'host_id'                         => $this->Service->host_id,
                        'service_id'                      => $this->Service->id,
                        'sla_id'                          => $this->Sla['id'],
                        'evaluation_total_time'           => $evaluationTotalTime,
                        'total_time'                      => 0,
                        'evaluation_start'                => date('Y-m-d H:i:s', $currentStart->getTimestamp()),
                        'evaluation_end'                  => date('Y-m-d H:i:s', $currentStart->getTimestamp() + 24 * 3600),
                        'minimal_availability_time'       => 0,
                        'determined_availability_time'    => 0,
                        'minimal_availability_percent'    => $this->Sla['minimal_availability'],
                        'determined_availability_percent' => $availabilityInPercent,
                        'determined_outage_time'          => 0,
                        'determined_number_outages'       => $totalNumberOfOutages,
                        'ok'                              => 0,
                        'warning'                         => 0,
                        'critical'                        => 0,
                        'unknown'                         => 0,
                        'sla_service_outages'             => []
                    ]);
                }
                $currentStart->modify('next day');
                $i++;
            }
        }

        $errors = [];
        $statusErrors = [];
        if (!empty($slaAvailabilityStatusServicesLog)) {
            if ($SlaAvailabilityStatusServicesLogTable->saveMany($slaAvailabilityStatusServicesLog)) {
                //successfully finished
                //save last log data in sla service status table

                $lastLogEntry = $slaAvailabilityStatusServicesLog[sizeof($slaAvailabilityStatusServicesLog) - 1];
                $lastLogArray = $lastLogEntry->toArray();
                foreach (['id', 'created', 'updated'] as $unwantedKey) {
                    if (isset($lastLogArray[$unwantedKey])) {
                        unset($lastLogArray[$unwantedKey]);
                    }
                }

                if ($this->Service->get('sla_availability_status_service') !== null) {
                    $entity = $this->Service->get('sla_availability_status_service');

                    $entity = $SlaAvailabilityStatusServicesTable->patchEntity($entity, $lastLogArray);
                } else {
                    $entity = $SlaAvailabilityStatusServicesTable->newEntity($lastLogArray);
                }


                $SlaAvailabilityStatusServicesTable->save($entity);
                if ($entity->hasErrors()) {
                    $statusErrors = $entity->getErrors();
                }

                return new SlaResult(empty($statusErrors), [], $statusErrors);
            }

            foreach ($slaAvailabilityStatusServicesLog as $entity) {
                $errors[] = $entity->getErrors();
            }
        }

        $success = empty($errors) && empty($statusErrors);
        return new SlaResult($success, $errors, $statusErrors);
    }

    /**
     * @param $start
     * @param $end
     * @return array
     * @throws \Exception
     */
    private function getTimeSlices($start, $end) {
        $timeSlicesGlobal = Hash::insert(
            SlaDaterangesCreator::createDateRangesForSlaForOneDay(
                $start,
                $this->Sla['timeperiod']['timeperiod_timeranges']
            ),
            '{n}.is_downtime', false
        );

        $timeSlicesCounter = sizeof($timeSlicesGlobal);

        if (!empty($timeSlicesGlobal)) {
            if ($timeSlicesGlobal[0]['start'] < $start) {
                $timeSlicesGlobal[0]['start'] = $start;
            }
            if ($timeSlicesGlobal[$timeSlicesCounter - 1]['end'] > $end) {
                $timeSlicesGlobal[$timeSlicesCounter - 1]['end'] = $end;
            }
        }

        if ($this->Sla['timeperiod']['calendar_id'] === 0) {
            return $timeSlicesGlobal;
        }
        //filter holidays if needed
        if (!empty($this->Sla['timeperiod']['calendar']['calendar_holidays'])) {
            $holidays = Hash::extract($this->Sla['timeperiod']['calendar']['calendar_holidays'], '{n}.date');
            foreach ($timeSlicesGlobal as $key => $timeSliceGlobal) {
                if (in_array(date('Y-m-d', $timeSliceGlobal['start']), $holidays, true)) {
                    unset($timeSlicesGlobal[$key]);
                }
            }
            if (empty($timeSlicesGlobal)) {
                throw new \Exception(__(
                    'Time evaluation sla data is not available for the selected period ({0} with calendar: {1})',
                    $this->Sla['timeperiod']['name'],
                    $this->Sla['timeperiod']['calendar']['name']
                ));
            }
        }

        return $timeSlicesGlobal;
    }


    private function debugDateRanges(array $dateRanges) {
        echo PHP_EOL;
        foreach ($dateRanges as $range) {
            if (isset($range['entryType'])) {
                // Timerange
                printf('Start: %s  End: %s  entryType: %s%s',
                    date('l d.m.Y H:i:s', $range['start']),
                    date('l d.m.Y H:i:s', $range['end']),
                    $range['entryType'],
                    PHP_EOL
                );
            } else {
                // Timeslice (24x7, 8x5)
                printf('Start: %s  End: %s %s',
                    date('l d.m.Y H:i:s', $range['start']),
                    date('l d.m.Y H:i:s', $range['end']),
                    PHP_EOL
                );
            }
        }
    }
}
