<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


declare(strict_types=1);

namespace SLAModule\Model\Table;

use App\Lib\Traits\PluginManagerTableTrait;
use App\Model\Table\HostsTable;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\RepositoryInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use SLAModule\Model\Entity\SlaHostOutage;

/**
 * SlaHostOutages Model
 *
 * @property HostsTable&BelongsTo $Hosts
 *
 * @method SlaHostOutage newEmptyEntity()
 * @method SlaHostOutage newEntity(array $data, array $options = [])
 * @method SlaHostOutage[] newEntities(array $data, array $options = [])
 * @method SlaHostOutage get($primaryKey, $options = [])
 * @method SlaHostOutage findOrCreate($search, ?callable $callback = null, $options = [])
 * @method SlaHostOutage patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method SlaHostOutage[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method SlaHostOutage|false save(EntityInterface $entity, $options = [])
 * @method SlaHostOutage saveOrFail(EntityInterface $entity, $options = [])
 * @method SlaHostOutage[]|ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method SlaHostOutage[]|ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method SlaHostOutage[]|ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method SlaHostOutage[]|ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class SlaHostOutagesTable extends Table {

    use PluginManagerTableTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('sla_host_outages');
        $this->setDisplayField(['host_id', 'start_time']);
        $this->setPrimaryKey(['host_id', 'start_time']);

        $this->belongsTo('Hosts', [
            'foreignKey' => 'host_id',
            'joinType'   => 'INNER',
            'className'  => 'Hosts',
        ]);

        $this->belongsTo('SlaAvailabilityStatusHostsLog', [
            'foreignKey' => 'sla_availability_status_hosts_log_id',
            'joinType'   => 'INNER',
            'className'  => 'SLAModule.SlaAvailabilityStatusHostsLog',
        ]);
    }

    /**
     * Bind Core2Plugin associations
     *
     * @param RepositoryInterface $coreTable
     */
    public function bindCoreAssociations(RepositoryInterface $coreTable) {
        switch ($coreTable->getAlias()) {
            case 'Hosts':
                $coreTable->hasMany('SlaHostOutages', [
                    'className'  => 'SLAModule.SlaHostOutages',
                    'dependent'  => true,
                    'foreignKey' => 'host_id'
                ]);
                break;
        }
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('sla_id')
            ->requirePresence('sla_id', 'create')
            ->notEmptyString('sla_id');

        $validator
            ->integer('sla_availability_status_hosts_log_id')
            ->requirePresence('sla_availability_status_hosts_log_id', 'create')
            ->notEmptyString('sla_availability_status_hosts_log_id');

        $validator
            ->requirePresence('end_time', 'create')
            ->notEmptyString('end_time');

        $validator
            ->scalar('output')
            ->maxLength('output', 1024)
            ->allowEmptyString('output');

        $validator
            ->boolean('is_hardstate')
            ->notEmptyString('is_hardstate');

        $validator
            ->boolean('in_downtime')
            ->notEmptyString('in_downtime');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn('host_id', 'Hosts'), ['errorField' => 'host_id']);

        return $rules;
    }
}
