<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


declare(strict_types=1);

namespace SLAModule\Model\Table;

use App\Lib\Traits\PluginManagerTableTrait;
use App\Model\Table\HostsTable;
use App\Model\Table\ServicesTable;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\RepositoryInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use SLAModule\Model\Entity\SlaServiceOutage;

/**
 * SlaServiceOutages Model
 *
 * @property HostsTable&BelongsTo $Hosts
 * @property ServicesTable&BelongsTo $Services
 *
 * @method SlaServiceOutage newEmptyEntity()
 * @method SlaServiceOutage newEntity(array $data, array $options = [])
 * @method SlaServiceOutage[] newEntities(array $data, array $options = [])
 * @method SlaServiceOutage get($primaryKey, $options = [])
 * @method SlaServiceOutage findOrCreate($search, ?callable $callback = null, $options = [])
 * @method SlaServiceOutage patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method SlaServiceOutage[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method SlaServiceOutage|false save(EntityInterface $entity, $options = [])
 * @method SlaServiceOutage saveOrFail(EntityInterface $entity, $options = [])
 * @method SlaServiceOutage[]|ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method SlaServiceOutage[]|ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method SlaServiceOutage[]|ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method SlaServiceOutage[]|ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class SlaServiceOutagesTable extends Table {

    use PluginManagerTableTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('sla_service_outages');
        $this->setDisplayField(['service_id', 'start_time']);
        $this->setPrimaryKey(['service_id', 'start_time']);

        $this->belongsTo('Hosts', [
            'foreignKey' => 'host_id',
            'joinType'   => 'INNER',
            'className'  => 'SLAModule.Hosts',
        ]);
        $this->belongsTo('Services', [
            'foreignKey' => 'service_id',
            'joinType'   => 'INNER',
            'className'  => 'SLAModule.Services',
        ]);
        $this->belongsTo('SlaAvailabilityStatusServicesLog', [
            'foreignKey' => 'sla_availability_status_services_log_id',
            'joinType'   => 'INNER',
            'className'  => 'SLAModule.SlaAvailabilityStatusServicesLog',
        ]);
    }

    /**
     * Bind Core2Plugin associations
     *
     * @param RepositoryInterface $coreTable
     */
    public function bindCoreAssociations(RepositoryInterface $coreTable) {
        switch ($coreTable->getAlias()) {
            case 'Services':
                $coreTable->hasMany('SlaServiceOutages', [
                    'className'  => 'SLAModule.SlaServiceOutages',
                    'dependent'  => true,
                    'foreignKey' => 'service_id'
                ]);
                break;
        }
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('host_id')
            ->notEmptyString('host_id');

        $validator
            ->integer('service_id')
            ->notEmptyString('service_id');

        $validator
            ->integer('sla_id')
            ->requirePresence('sla_id', 'create')
            ->notEmptyString('sla_id');

        $validator
            ->integer('sla_availability_status_services_log_id')
            ->requirePresence('sla_availability_status_services_log_id', 'create')
            ->notEmptyString('sla_availability_status_services_log_id');

        $validator
            ->requirePresence('end_time', 'create')
            ->notEmptyString('end_time');

        $validator
            ->scalar('output')
            ->maxLength('output', 1024)
            ->allowEmptyString('output');

        $validator
            ->boolean('is_hardstate')
            ->notEmptyString('is_hardstate');

        $validator
            ->boolean('in_downtime')
            ->notEmptyString('in_downtime');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn('host_id', 'Hosts'), ['errorField' => 'host_id']);
        $rules->add($rules->existsIn('service_id', 'Services'), ['errorField' => 'service_id']);

        return $rules;
    }
}
