<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace SLAModule\Model\Table;

use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Utility\Hash;
use Cake\Validation\Validator;

/**
 * SlaSettings Model
 *
 * @method \SLAModule\Model\Entity\SlaSetting newEmptyEntity()
 * @method \SLAModule\Model\Entity\SlaSetting newEntity(array $data, array $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[] newEntities(array $data, array $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting get($primaryKey, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \SLAModule\Model\Entity\SlaSetting[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class SlaSettingsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('sla_settings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('max_age')
            ->greaterThan('max_age', 0)
            ->notEmptyString('max_age');

        return $validator;
    }

    /**
     * Returns the current global SLA settings as array
     * @return array
     */
    public function getSettings() {
        $defaults = [
            'max_age' => 53
        ];

        try {
            $entity = $this->find()->firstOrFail();

            $settings = Hash::merge(
                $defaults,
                $entity->toArray()
            );

            return $settings;
        } catch (RecordNotFoundException $e) {
            return $defaults;
        }
    }

    /**
     * @return array|\Cake\Datasource\EntityInterface|\SLAModule\Model\Entity\SlaSetting
     */
    public function getSettingsEntity() {
        try {
            return $this->find()->firstOrFail();
        } catch (RecordNotFoundException $e) {
            return $this->newEmptyEntity();
        }
    }
}
