<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace SLAModule\Reports;

use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\Reports\DaterangesCreator;

/**
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * !! WHENEVER YOU CHANGE THIS FILE - RUN THE TESTS                                                 !!
 * !! cd /opt/openitc/frontend/plugins/SLAModule                                                    !!
 * !! vendor/phpunit/phpunit/phpunit tests/TestCase/Command/SlaDaterangesCreatorCommandTest.php     !!
 * !!                                                                                               !!
 * !! SEE SlaDaterangesCreatorCommandTest.php for more information!                                 !!
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 */
class SlaDaterangesCreator extends DaterangesCreator {


    /**
     * @param int $lastUpdateDate
     * @param string $interval
     * @return array
     * @throws \Exception
     */
    public static function createDateSlicesForSlaByIntervalAndLastUpdateDate($lastUpdateDate, $interval, $forceNew = false) {
        $now = time();
        $dateTimeSlices = [];

        // If this is the first point of the evaluation period entryType is set to new (01.11.2022), or if the date is in the middle of the evaluation period (06.11.2022) entryType = update
        $initialEntryType = ($forceNew === false) ? 'update' : 'new';

        $LastUpdateDate = new \DateTime(date('Y-m-d H:i:s', $lastUpdateDate));
        $LastUpdateDate->setTime(0, 0, 0);
        $lastUpdateDate = $LastUpdateDate->getTimestamp();
        $h = $LastUpdateDate->format('H');
        $m = $LastUpdateDate->format('i');
        $s = $LastUpdateDate->format('s');
        switch ($interval) {
            case 'DAY':
                if ($h == 0 && $m == 0 && $s == 0) {
                    $initialEntryType = 'new'; //midnight -> new day detected
                }
                if (date('zY', $lastUpdateDate) !== date('zY', $now)) {
                    $nextDay = $LastUpdateDate->modify('next day');
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => strtotime($nextDay->format('d.m.Y') . ' 00:00:00'),
                        'entryType' => $initialEntryType
                    ];
                    $end = new \DateTime(date('d.m.Y', $now));
                    $end = $end->modify('+1 day');

                    $dateInterval = new \DateInterval('P1D');
                    $dateRange = new \DatePeriod($nextDay, $dateInterval, $end);
                    /** @var \DateTime $date */
                    foreach ($dateRange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $dateFromTimestamp = new \DateTime();
                        $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                        $dateFromTimestamp->setTime(0, 0, 0);
                        if (date('z', $dateAsTimestamp) < date('z', $now)) {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => $dateFromTimestamp->modify('+1 day')->getTimestamp(),
                                'entryType' => 'new'
                            ];
                        } else {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => $now,
                                'entryType' => 'new'
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => $now,
                        'entryType' => $initialEntryType
                    ];
                }
                return $dateTimeSlices;
            case 'WEEK':
                $w = date('w', $lastUpdateDate); // ISO-8601 week number of year, weeks starting on Monday(=1)

                if ($h == 0 && $m == 0 && $s == 0 && $w == 1) {
                    $initialEntryType = 'new'; //midnight and monday -> new day detected
                }

                if (date('WY', $lastUpdateDate) !== date('WY', $now)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $lastUpdateDate));
                    $begin->setTime(0, 0, 0);
                    $nextWeekMonday = $begin->modify('monday next week');
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => strtotime($nextWeekMonday->format('d.m.Y') . ' 00:00:00'),
                        'entryType' => $initialEntryType
                    ];
                    $end = new \DateTime(date('Y-m-d', $now));

                    $interval = new \DateInterval('P1W');
                    $dateRange = new \DatePeriod($nextWeekMonday, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($dateRange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $dateFromTimestamp = new \DateTime();
                        $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                        $dateFromTimestamp->setTime(0, 0, 0);

                        if (date('W', $now) === date('W', $dateAsTimestamp)) {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => $now,
                                'entryType' => 'new'
                            ];
                        } else {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => strtotime(date('d.m.Y', $dateFromTimestamp->getTimestamp()) . ' monday next week 00:00:00'),
                                'entryType' => 'new'
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => $now,
                        'entryType' => $initialEntryType
                    ];
                }
                return $dateTimeSlices;
                break;
            case 'MONTH':
                $j = date('j', $lastUpdateDate); //Day of the month without leading zeros
                if ($h == 0 && $m == 0 && $s == 0 && $j == 1) {
                    $initialEntryType = 'new'; //midnight and first day of month -> new day detected
                }
                if (date('mY', $lastUpdateDate) !== date('mY', $now)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $lastUpdateDate));
                    $firstDayOfNextMonth = $begin->modify('first day of next month');
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => strtotime($firstDayOfNextMonth->format('d.m.Y') . ' 00:00:00'),
                        'entryType' => $initialEntryType
                    ];

                    $nextMonthFirstDay = $begin->modify('first day of this month 00:00:00');
                    $end = new \DateTime(date('Y-m-d', $now));

                    $interval = new \DateInterval('P1M');
                    $dateRange = new \DatePeriod($nextMonthFirstDay, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($dateRange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $dateFromTimestamp = new \DateTime();
                        $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                        $dateFromTimestamp->setTime(0, 0, 0);

                        if (date('m', $now) === date('m', $dateAsTimestamp)) {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => $now,
                                'entryType' => 'new'
                            ];
                        } else {
                            $dateTimeSlices[] = [
                                'start'     => $dateFromTimestamp->getTimestamp(),
                                'end'       => strtotime(date('d.m.Y', $dateFromTimestamp->getTimestamp()) . ' first day of next month 00:00:00'),
                                'entryType' => 'new'
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => $now,
                        'entryType' => $initialEntryType
                    ];
                }
                return $dateTimeSlices;
            case 'QUARTER':
                $numberFromCurrentQuarter = self::getNumberFromQuarter($now);
                $firstDayOfThisQuarter = self::firstDayOfQuarter($LastUpdateDate, $numberFromCurrentQuarter);

                if ($firstDayOfThisQuarter->getTimestamp() === $lastUpdateDate) {
                    $initialEntryType = 'new';
                }

                if (date('Y', $lastUpdateDate) . self::getNumberFromQuarter($lastUpdateDate) !== date('Y', $now) . $numberFromCurrentQuarter) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $lastUpdateDate));
                    $lastUpdateQuarter = self::getNumberFromQuarter($lastUpdateDate);
                    $lastDayOfThisQuarter = self::lastDayOfQuarter($begin, $lastUpdateQuarter);
                    $nextQuarterFirstDay = $lastDayOfThisQuarter->modify('tomorrow 00:00:00');
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => strtotime($nextQuarterFirstDay->format('d.m.Y') . ' 00:00:00'),
                        'entryType' => $initialEntryType
                    ];

                    $end = new \DateTime(date('Y-m-d', $now));
                    $interval = new \DateInterval('P3M');
                    $daterange = new \DatePeriod($nextQuarterFirstDay, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($daterange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $numberFromDateQuarter = self::getNumberFromQuarter($dateAsTimestamp);
                        if (date('Y', $now) . $numberFromCurrentQuarter === date('Y', $dateAsTimestamp) . self::getNumberFromQuarter($dateAsTimestamp)) {
                            $dateTimeSlices[] = [
                                'start'     => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                'end'       => $now,
                                'entryType' => 'new'
                            ];
                        } else {
                            $end = $date;
                            $firstDayOfThisQuarter = self::lastDayOfQuarter($end, $numberFromDateQuarter)->modify('tomorrow 00:00:00');
                            $dateTimeSlices[] = [
                                'start'     => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                'end'       => $firstDayOfThisQuarter->getTimestamp(),
                                'entryType' => 'new'
                            ];
                        }
                    }

                } else {
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => $now,
                        'entryType' => $initialEntryType
                    ];
                }
                return $dateTimeSlices;
            case 'YEAR':
                $j = date('j', $lastUpdateDate); //Day of the month without leading zeros
                $n = date('n', $lastUpdateDate); //Numeric representation of a month, without leading zeros
                if ($h == 0 && $m == 0 && $s == 0 && $j == 1 && $n == 1) {
                    $initialEntryType = 'new'; //midnight and first day of the year -> new day detected
                }
                if (date('Y', $lastUpdateDate) !== date('Y', $now)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $lastUpdateDate));
                    $firstDayNextYear = $begin->modify('1 January ' . (date('Y', $lastUpdateDate) + 1) . ' 00:00:00');
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => strtotime(date('d.m.Y', $firstDayNextYear->getTimestamp()) . ' 00:00:00'),
                        'entryType' => $initialEntryType
                    ];
                    $end = new \DateTime(date('Y-m-d', $now));

                    $interval = new \DateInterval('P1Y');
                    $daterange = new \DatePeriod($firstDayNextYear, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($daterange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $dateFromTimestamp = new \DateTime();
                        $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                        $dateFromTimestamp->setTime(0, 0, 0);

                        if (date('Y', $now) === date('Y', $dateAsTimestamp)) {
                            $dateTimeSlices[] = [
                                'start'     => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                'end'       => $now,
                                'entryType' => 'new'
                            ];
                        } else {
                            $dateTimeSlices[] = [
                                'start'     => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                'end'       => strtotime('1 January ' . (date('Y', $dateAsTimestamp) + 1) . ' 00:00:00'),
                                'entryType' => 'new'
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'start'     => $lastUpdateDate,
                        'end'       => $now,
                        'entryType' => $initialEntryType
                    ];
                }
                return $dateTimeSlices;
        }
        return $dateTimeSlices;
    }

    /**
     * @param int $date_start_timestamp
     * @param int $date_end_timestamp
     * @param array $time_ranges
     * @return array
     */
    public static function createDateRangesForSla($date_start_timestamp, $date_end_timestamp, $time_ranges = [], $calendar = []) {
        $time_slices_default = [];
        $time_slices = [];
        $first_monday_in_week = strtotime(date('d.m.Y H:i:s', $date_start_timestamp) . ' -' . (date('N', $date_start_timestamp) - 1) . ' days');
        //evaluation period in weeks
        $week_count = ceil(($date_end_timestamp - $first_monday_in_week) / (3600 * 24 * 7));
        $default_week_created = false;

        $time_ranges = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');//group by day

        for ($week = 0; $week <= $week_count; $week++) {
            if (!$default_week_created) {
                for ($day = 0; $day < 7; $day++) {
                    //start is always 00:00

                    $current_day_timestamp = strtotime('+ ' . $day . ' day', $first_monday_in_week);
                    $day_of_week = date('N', $current_day_timestamp);
                    if (array_key_exists($day_of_week, $time_ranges)) {
                        foreach ($time_ranges[$day_of_week] as $time_range) {
                            $duration_in_seconds = (strtotime($time_range['end']) - strtotime($time_range['start']));

                            $time_slices_default[] = [
                                'start' => strtotime(date('d.m.Y ', $current_day_timestamp) . ' ' . $time_range['start']),
                                'end'   => strtotime(date('d.m.Y ', $current_day_timestamp) . ' ' . $time_range['start'] . ' +' . $duration_in_seconds . ' seconds'),
                            ];
                        }
                    }
                }
                $default_week_created = true;
                $time_slices = $time_slices_default;
                continue;
            }
            foreach ($time_slices_default as $time_slice) {
                $time_slices[] = [
                    'start' => strtotime(date('d.m.Y H:i:s', $time_slice['start']) . ' + ' . $week . ' week'),
                    'end'   => strtotime(date('d.m.Y H:i:s', $time_slice['end']) . ' + ' . $week . ' week'),
                ];
            }
        }

        $timeSlicesFiltered = self::removeUselessTimeslicesInSla(
            date('Ymd', $date_start_timestamp),
            date('Ymd', $date_end_timestamp - 3600), $time_slices
        );
        if (empty($calendar['calendar_holidays'])) {
            return $timeSlicesFiltered;
        }
        $holidays = Hash::extract($calendar['calendar_holidays'], '{n}.date');
        foreach ($timeSlicesFiltered as $key => $timeSliceGlobal) {
            if (in_array(date('Y-m-d', $timeSliceGlobal['start']), $holidays, true)) {
                unset($timeSlicesFiltered[$key]);
            }
        }

        return $timeSlicesFiltered;
    }

    /**
     * @param $date_timestamp
     * @param $time_ranges
     * @return array
     */
    public static function createDateRangesForSlaForOneDay($date_timestamp, $time_ranges = []) {
        $time_slices = [];

        $time_ranges = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');//group by day

        $day_of_week = date('N', $date_timestamp);
        if (array_key_exists($day_of_week, $time_ranges)) {
            foreach ($time_ranges[$day_of_week] as $time_range) {
                $duration_in_seconds = (strtotime($time_range['end']) - strtotime($time_range['start']));
                $time_slices[] = [
                    'start' => strtotime(date('d.m.Y ', $date_timestamp) . ' ' . $time_range['start']),
                    'end'   => strtotime(date('d.m.Y ', $date_timestamp) . ' ' . $time_range['start'] . ' +' . $duration_in_seconds . ' seconds'),
                ];
            }
        }

        return $time_slices;

    }

    /**
     * @param $sla_interval
     * @return int timestamp
     * @throws \Exception
     */
    public static function getEndOfRangeTimeBySlaInterval($date, $sla_interval) {
        $date = new \DateTime(date('d.m.Y H:i', $date));
        switch ($sla_interval) {
            case 'DAY':
                $date->modify('today');
                break;
            case 'WEEK':
                $date->modify('this Sunday');
                break;
            case 'MONTH':
                $date->modify('last day of this month');
                break;
            case 'YEAR':
                $date->modify('last day of last month');
                break;
            case 'QUARTER':
                $currentMonth = date('n');
                $currentYear = date('o');
                if ($currentMonth >= 1 && $currentMonth <= 3) {
                    $date->modify('last day of december ' . ($currentYear - 1));
                } else if ($currentMonth >= 4 && $currentMonth <= 6) {
                    $date->modify('last day of march ' . $currentYear);
                } else if ($currentMonth >= 7 && $currentMonth <= 9) {
                    $date->modify('last day of june ' . $currentYear);
                } else if ($currentMonth >= 10 && $currentMonth <= 12) {
                    $date->modify('last day of september ' . $currentYear);
                }
                break;
        }

        $date->setTime(24, 00, 00);
        return $date->getTimestamp();
    }

    /**
     * @param $date_start string Ymd
     * @param $date_end string Ymd
     * @param $time_slices array
     * @return array
     */
    public static function removeUselessTimeslicesInSla($date_start, $date_end, $time_slices) {
        $time_slices_new = [];
        foreach ($time_slices as $time_slice) {
            $current_time_slice_start = date('Ymd', $time_slice['start']);
            if ($current_time_slice_start < $date_start) {
                continue;
            }

            if ($current_time_slice_start > $date_end) {
                return $time_slices_new;
            }
            $time_slices_new[] = $time_slice;
        }

        return $time_slices_new;
    }


    /**
     * @param $report_interval
     * @param $lastEvaluationDate
     * @param $slaStartDate
     * @return array|int[]
     * @throws \Exception
     */
    public static function getDateDetailsForInterval($report_interval, $lastEvaluationDate, $slaStartDate = null) {
        $useSlaStartDateTime = false;
        $evaluationDate = new \DateTime(date('d.m.Y H:i:s', $lastEvaluationDate));
        $evaluationDate->setTime(0, 0, 0);

        if ($slaStartDate) {
            $slaStartDate = new \DateTime(date('d.m.Y H:i:s', $slaStartDate));

        }

        $dateDetails = [
            'start'       => 0,
            'end'         => 0,
            'days_passed' => 0,
            'days_left'   => 0,
            'total_days'  => 0
        ];

        switch ($report_interval) {
            case 'DAY':
                $lastDay = new \DateTime(date('d.m.Y H:i:s', $lastEvaluationDate));
                $lastDay->setTime(24, 0, 0);

                $dateDetails = [
                    'start'       => $evaluationDate->getTimestamp(),
                    'end'         => $lastDay->getTimestamp(),
                    'days_passed' => 1,
                    'days_left'   => 0,
                    'total_days'  => 1
                ];
                break;
            case 'WEEK':
                $firstDay = $evaluationDate->modify('last Monday');
                $firstDay->setTime(0, 0, 0);
                $lastDay = $firstDay->add(new \DateInterval('P7D'));

                $dateDetails = [
                    'start'       => $firstDay->getTimestamp(),
                    'end'         => $lastDay->getTimestamp(),
                    'days_passed' => (int)date('N', $lastEvaluationDate),
                    'days_left'   => 7 - (int)date('N', $lastEvaluationDate),
                    'total_days'  => 7
                ];
                break;
            case 'MONTH':
                $firstDay = $evaluationDate->modify('first day of this month');
                $firstDay->setTime(0, 0, 0);
                $firstDayTimestamp = $firstDay->getTimestamp();

                $lastDay = $evaluationDate->modify('last day of this month');
                $dateDetails = [
                    'start'       => $firstDayTimestamp,
                    'end'         => $lastDay->getTimestamp(),
                    'days_passed' => (int)date('d', $lastEvaluationDate),
                    'days_left'   => date('d', $lastDay->getTimestamp()) - (int)date('d', $lastEvaluationDate),
                    'total_days'  => (int)$firstDay->format('t')
                ];
                break;
            case 'QUARTER':
                $currentMonth = (int)date('n', $evaluationDate->getTimestamp());
                $currentYear = (int)date('o', $evaluationDate->getTimestamp());

                $currentQuarter = (int)ceil($currentMonth / 3);
                switch ($currentQuarter) {
                    case 1:
                        $firstDay = strtotime('1 january ' . $currentYear . ' 00:00:00');
                        $lastDay = strtotime('31 march ' . $currentYear . ' 24:00:00');
                        break;
                    case 2:
                        $firstDay = strtotime('1 april ' . $currentYear . ' 00:00:00');
                        $lastDay = strtotime('30 june ' . $currentYear . ' 24:00:00');
                        break;
                    case 3:
                        $firstDay = strtotime('1 juli ' . $currentYear . ' 00:00:00');
                        $lastDay = strtotime('30 september ' . $currentYear . ' 24:00:00');
                        break;
                    case 4:
                        $firstDay = strtotime('1 october ' . $currentYear . ' 00:00:00');
                        $lastDay = strtotime('31 December ' . $currentYear . ' 24:00:00');

                        break;

                }
                $firstDay = new \DateTime(date('d.m.Y H:i:s', $firstDay));
                $lastDay = new \DateTime((date('d.m.Y H:i:s', $lastDay)));


                $daysPassed = $firstDay->diff($evaluationDate);
                $daysLeft = $evaluationDate->diff($lastDay);
                $totalDays = $firstDay->diff($lastDay);

                $dateDetails = [
                    'start'       => $firstDay->getTimestamp(),
                    'end'         => $lastDay->getTimestamp(),
                    'days_passed' => (int)$daysPassed->format('%a'),
                    'days_left'   => (int)$daysLeft->format('%a'),
                    'total_days'  => (int)$totalDays->format('%a')
                ];

                break;
            case 'YEAR':
                $currentYear = (int)date('o', $evaluationDate->getTimestamp());

                $firstDay = strtotime('1 January  ' . $currentYear . ' 00:00:00');
                $lastDay = strtotime('31 December ' . $currentYear . ' 24:00:00');


                $firstDay = new \DateTime(date('d.m.Y H:i:s', $firstDay));
                $lastDay = new \DateTime((date('d.m.Y H:i:s', $lastDay)));

                $daysPassed = $firstDay->diff($evaluationDate);
                $daysLeft = $evaluationDate->diff($lastDay);

                $dateDetails = [
                    'start'       => $firstDay,
                    'end'         => $lastDay->getTimestamp(),
                    'days_passed' => (int)$daysPassed->format('%a'),
                    'days_left'   => (int)$daysLeft->format('%a'),
                    'total_days'  => ($firstDay->format('L')) ? 366 : 365
                ];

                break;
        }
        return $dateDetails;
    }

    /**
     * @param $interval
     * @param $startDate
     * @param $endDate
     * @return array
     * @throws \Exception
     */
    public static function createDateSlicesByIntervalAndStartAndEndDate($interval, $startDate, $endDate) {
        $dateTimeSlices = [];

        $startDate = new \DateTime(date('Y-m-d H:i:s', $startDate));
        $startDate->setTime(0, 0, 0);
        $startDate = $startDate->getTimestamp();
        $nowMidnight = strtotime(date('d.m.Y', time()) . ' 00:00:00');
        if ($startDate >= $nowMidnight) {
            $startDate = strtotime('yesterday 00:00:00');
            $endDate = strtotime('yesterday 24:00:00');
        } else if ($endDate > $nowMidnight) {
            $endDate = strtotime('yesterday 00:00:00');
        }

        switch ($interval) {
            case 'DAY':
                $weekdays = [
                    1 => __('Monday'),
                    2 => __('Tuesday'),
                    3 => __('Wednesday'),
                    4 => __('Thursday'),
                    5 => __('Friday'),
                    6 => __('Saturday'),
                    7 => __('Sunday'),
                ];
                $endDate = new \DateTime(date('Y-m-d H:i:s', $endDate));
                $endDate->setTime(0, 0, 0);
                $endDate->modify('+ 1 day');
                $endDate = $endDate->getTimestamp();

                if (date('zY', $startDate) !== date('zY', $endDate)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $startDate));
                    $end = new \DateTime(date('d.m.Y H:i:s', $endDate));
                    $dateInterval = new \DateInterval('P1D');
                    $dateRange = new \DatePeriod($begin, $dateInterval, $end);
                    /** @var \DateTime $date */
                    foreach ($dateRange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        if ($dateAsTimestamp < $nowMidnight) {
                            $dateTimeSlices[] = [
                                'title' => $weekdays[date('N', $dateAsTimestamp)],
                                'start' => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                'end'   => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 24:00:00')
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'title' => $weekdays[date('N', $startDate)],
                        'start' => $startDate,
                        'end'   => $endDate
                    ];
                }
                return $dateTimeSlices;
            case 'WEEK':
                if (date('WY', $startDate) !== date('WY', $endDate)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $startDate));
                    $sunday = $begin->modify('sunday this week');
                    $dateTimeSlices[] = [
                        'title' => __('Week {0}', date('W [Y]', $startDate)),
                        'start' => $startDate,
                        'end'   => strtotime($sunday->format('d.m.Y') . ' 24:00:00')
                    ];
                    $nextWeekMonday = $begin->modify('monday next week 00:00:00');
                    $end = new \DateTime(date('Y-m-d', $endDate) . ' 24:00:00');

                    $interval = new \DateInterval('P1W');
                    $daterange = new \DatePeriod($nextWeekMonday, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($daterange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        if ($dateAsTimestamp < $nowMidnight) {
                            $dateFromTimestamp = new \DateTime();
                            $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                            $dateFromTimestamp->setTime(0, 0, 0);
                            $dateFromTimestamp = $dateFromTimestamp->getTimestamp();

                            if (date('W', $endDate) === date('W', $dateAsTimestamp)) {
                                $dateTimeSlices[] = [
                                    'title' => __('Week {0}', date('W [Y]', $dateFromTimestamp)),
                                    'start' => $dateFromTimestamp,
                                    'end'   => strtotime(date('d.m.Y', $endDate) . ' 24:00:00')
                                ];
                            } else {
                                $dateTimeSlices[] = [
                                    'title' => __('Week {0}', date('W [Y]', $dateFromTimestamp)),
                                    'start' => $dateFromTimestamp,
                                    'end'   => strtotime(date('d.m.Y', $dateFromTimestamp) . ' monday next week 00:00:00')
                                ];
                            }
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'title' => __('Week {0}', date('W [Y]', $startDate)),
                        'start' => $startDate,
                        'end'   => $endDate
                    ];
                }
                return $dateTimeSlices;
            case 'MONTH':
                $months = [
                    1  => __('January'),
                    2  => __('February'),
                    3  => __('March'),
                    4  => __('April'),
                    5  => __('May'),
                    6  => __('June'),
                    7  => __('July'),
                    8  => __('August'),
                    9  => __('September'),
                    10 => __('October'),
                    11 => __('November'),
                    12 => __('December')
                ];
                if (date('mY', $startDate) !== date('mY', $endDate)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $startDate));
                    $firstDayOfNextMonth = $begin->modify('first day of next month');
                    $dateTimeSlices[] = [
                        'title' => sprintf('%s [%s]', $months[date('n', $startDate)], date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => strtotime($firstDayOfNextMonth->format('d.m.Y') . ' 00:00:00')
                    ];

                    $nextMonthFirstDay = $begin->modify('first day of this month 00:00:00');
                    $end = new \DateTime(date('Y-m-d', $endDate) . ' 24:00:00');

                    $interval = new \DateInterval('P1M');
                    $dateRange = new \DatePeriod($nextMonthFirstDay, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($dateRange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $dateFromTimestamp = new \DateTime();
                        $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                        $dateFromTimestamp->setTime(0, 0, 0);
                        $dateFromTimestamp = $dateFromTimestamp->getTimestamp();

                        if (date('m', $endDate) === date('m', $dateAsTimestamp)) {
                            $dateTimeSlices[] = [
                                'title' => sprintf('%s [%s]', $months[date('n', $dateFromTimestamp)], date('Y', $dateFromTimestamp)),
                                'start' => $dateFromTimestamp,
                                'end'   => strtotime(date('d.m.Y', $endDate) . ' 24:00:00')
                            ];
                        } else {
                            $dateTimeSlices[] = [
                                'title' => sprintf('%s [%s]', $months[date('n', $dateFromTimestamp)], date('Y', $dateFromTimestamp)),
                                'start' => $dateFromTimestamp,
                                'end'   => strtotime(date('d.m.Y', $dateFromTimestamp) . ' first day of next month 00:00:00')
                            ];
                        }
                    }
                } else {
                    $dateTimeSlices[] = [
                        'title' => sprintf('%s [%s]', $months[date('n', $startDate)], date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => $endDate
                    ];
                }
                return $dateTimeSlices;
            case 'QUARTER':
                $numberFromCurrentQuarter = self::getNumberFromQuarter($endDate);
                $numberQuarterFromStart = self::getNumberFromQuarter($startDate);
                if (date('Y', $startDate) . self::getNumberFromQuarter($startDate) !== date('Y', $endDate) . $numberFromCurrentQuarter) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $startDate));
                    $lastUpdateQuarter = self::getNumberFromQuarter($startDate);
                    $lastDayOfThisQuarter = self::lastDayOfQuarter($begin, $lastUpdateQuarter);
                    $nextQuarterFirstDay = $lastDayOfThisQuarter->modify('tomorrow 00:00:00');
                    $dateTimeSlices[] = [
                        'title' => sprintf('%s %s [%s]', $numberQuarterFromStart, __('Quarter'), date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => strtotime($nextQuarterFirstDay->format('d.m.Y') . ' 00:00:00')
                    ];

                    $end = new \DateTime(date('Y-m-d', $endDate));
                    $interval = new \DateInterval('P3M');
                    $daterange = new \DatePeriod($nextQuarterFirstDay, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($daterange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        $numberQuarterFromStart = self::getNumberFromQuarter($dateAsTimestamp);
                        if ($dateAsTimestamp < $nowMidnight) {
                            $numberFromDateQuarter = self::getNumberFromQuarter($dateAsTimestamp);
                            if (date('Y', $endDate) . $numberFromCurrentQuarter === date('Y', $dateAsTimestamp) . self::getNumberFromQuarter($dateAsTimestamp)) {
                                $dateTimeSlices[] = [
                                    'title' => sprintf('%s %s [%s]', $numberQuarterFromStart, __('Quarter'), date('Y', $dateAsTimestamp)),
                                    'start' => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                    'end'   => strtotime(date('d.m.Y', $endDate) . ' 24:00:00')
                                ];
                            } else {
                                $end = $date;
                                $firstDayOfThisQuarter = self::lastDayOfQuarter($end, $numberFromDateQuarter)->modify('tomorrow 00:00:00');
                                $dateTimeSlices[] = [
                                    'title' => sprintf('%s %s [%s]', $numberQuarterFromStart, __('Quarter'), date('Y', $dateAsTimestamp)),
                                    'start' => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                    'end'   => $firstDayOfThisQuarter->getTimestamp()
                                ];
                            }
                        }
                    }
                } else {
                    $endDate = strtotime(date('d.m.Y', $endDate) . ' 24:00:00');
                    if ($endDate > $nowMidnight) {
                        $endDate = $nowMidnight;
                    }

                    $dateTimeSlices[] = [
                        'title' => sprintf('%s %s [%s]', self::getNumberFromQuarter($startDate), __('Quarter'), date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => $endDate
                    ];
                }
                return $dateTimeSlices;
            case 'YEAR':

                if (date('Y', $startDate) !== date('Y', $endDate)) {
                    $begin = new \DateTime(date('d.m.Y H:i:s', $startDate));
                    $firstDayNextYear = $begin->modify('1 January ' . (date('Y', $startDate) + 1) . ' 00:00:00');
                    $dateTimeSlices[] = [
                        'title' => sprintf('%s %s', __('Year'), date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => strtotime(date('d.m.Y', $firstDayNextYear->getTimestamp()) . ' 00:00:00')
                    ];
                    $end = new \DateTime(date('Y-m-d', $endDate));

                    $interval = new \DateInterval('P1Y');
                    $daterange = new \DatePeriod($firstDayNextYear, $interval, $end);
                    /** @var \DateTime $date */
                    foreach ($daterange as $date) {
                        $dateAsTimestamp = $date->getTimestamp();
                        if ($dateAsTimestamp < $nowMidnight) {
                            $dateFromTimestamp = new \DateTime();
                            $dateFromTimestamp->setTimestamp($dateAsTimestamp);
                            $dateFromTimestamp->setTime(0, 0, 0);

                            if (date('Y', $endDate) === date('Y', $dateAsTimestamp)) {
                                $dateTimeSlices[] = [
                                    'title' => sprintf('%s %s', __('Year'), date('Y', $dateAsTimestamp)),
                                    'start' => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                    'end'   => strtotime(date('d.m.Y', $endDate) . ' 24:00:00')
                                ];
                            } else {
                                $dateTimeSlices[] = [
                                    'title' => sprintf('%s %s', __('Year'), date('Y', $dateAsTimestamp)),
                                    'start' => strtotime(date('d.m.Y', $dateAsTimestamp) . ' 00:00:00'),
                                    'end'   => strtotime('1 January ' . (date('Y', $dateAsTimestamp) + 1) . ' 00:00:00')
                                ];
                            }
                        }
                    }
                } else {
                    $endDate = strtotime(date('d.m.Y', $endDate) . ' 24:00:00');
                    if ($endDate > $nowMidnight) {
                        $endDate = $nowMidnight;
                    }
                    $dateTimeSlices[] = [
                        'title' => sprintf('%s %s', __('Year'), date('Y', $startDate)),
                        'start' => $startDate,
                        'end'   => $endDate
                    ];
                }
                return $dateTimeSlices;
        }
    }

}
