<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.

/**
 * @var AppView $this
 * @var bool $success
 * @var array $report
 * @var UserTime $UserTime
 *
 */

use App\itnovum\openITCOCKPIT\Core\AngularJS\PdfAssets;
use App\View\AppView;
use itnovum\openITCOCKPIT\Core\Views\Logo;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

$Logo = new Logo();
$css = PdfAssets::getCssFiles();
?>
<head>
    <?php
    foreach ($css as $cssFile): ?>
        <link rel="stylesheet" type="text/css" href="<?= WWW_ROOT . $cssFile; ?>"/>
    <?php
    endforeach; ?>

    <style>
        <?= file_get_contents(PLUGIN . 'SLAModule' . DS . 'webroot' . DS . 'css/slas_pdf.css'); ?>
    </style>


</head>
<body class="">

<div class="row">
    <div class="col-6 padding-left-15 font-lg">
        <i class="fa fa-file-invoice" style="font-size: 20px!important;"></i>
        <?php echo h($report['sla']['name']); ?>
        <?php if (!empty($report['sla']['description'])): ?>
            <em><?= h($report['sla']['description']) ?></em>
        <?php endif; ?>
    </div>
    <div class="col-6">
        <img class="float-right" src="<?php echo $Logo->getLogoPdfPath(); ?>" width="200"/>
    </div>
</div>
<div class="row">
    <div class="col-4">
        <h4>
            <?= __('Analysis period'); ?>
        </h4>
    </div>
    <div class="col-8">
        <?php
        echo h($reportStart); ?>
        <i class="fas fa-long-arrow-alt-right"></i>
        <?php
        echo h($reportEnd);
        ?>
    </div>
    <div class="col-4">
        <h4>
            <?= __('Start date'); ?>
        </h4>
    </div>
    <div class="col-8">
        <?= $report['sla']['start_date']; ?>
    </div>

    <div class="col-4">
        <h4>
            <?= __('Evaluation interval'); ?>
        </h4>
    </div>
    <div class="col-8">
        <?= $report['sla']['evaluation_interval']; ?>
    </div>

    <div class="col-4">
        <h4>
            <?= __('Warning threshold as percentage'); ?>
        </h4>
    </div>
    <div class="col-8">
        <i class="fa-solid fa-triangle-exclamation text-warning"></i>
        <?= $report['sla']['warning_threshold']; ?>
    </div>

    <div class="col-4">
        <h4>
            <?= __('Guaranteed availability as percentage'); ?>
        </h4>
    </div>
    <div class="col-8">
        <i class="fa-solid fa-triangle-exclamation text-danger"></i>
        <?= $report['sla']['minimal_availability']; ?>
    </div>

</div>

<div class="row padding-top-10">
    <div class="col-4">
        <h4>
            <?= __('Evaluated time period'); ?>
        </h4>
    </div>

    <div class="col-8">
        <div class="row">
            <div class="col-12 bold text-primary">
                <?= h($report['timeperiod']['name']) ?>
            </div>
        </div>
        <?php foreach ($report['timeperiod']['timeperiod_timeranges'] as $timerange): ?>
            <div class="row">
                <div class="col-2">
                    <?= h($timerange['weekday']); ?>
                </div>
                <div class="col-10 text-primary">
                    <?= h($timerange['start']); ?>
                    -
                    <?= h($timerange['end']); ?>
                </div>
            </div>
        <?php endforeach; ?>
    </div>
</div>

<div class="row">
    <div class="col-4">
        <h4>
            <?= __('Host and service details'); ?>
        </h4>
    </div>
</div>
<div class="row">
    <div class="col-12">
        <?php foreach ($report['date_ranges'] as $key => $dateRange): ?>
            <div class="row html-report-host-row padding-top-20 padding-bottom-10 border-bottom border-primary">
                <div class="col-12 text-left font-weight-bold">
                    <i class="fa-solid fa-calendar-days text-primary"></i>
                    <?= $dateRange['title']; ?>,
                    <?= date('d.m.Y', $dateRange['start']); ?>
                </div>
            </div>
            <div class="row">
                <div class="col-12">
                    <?php foreach ($report['hosts'][$key] as $host): ?>
                        <div class="row break-inside-avoid">
                            <div class="col-8 bg-host">
                                <i class="fa-solid fa-desktop"></i>
                                <?= h($host['host']['name']); ?>
                            </div>

                            <div class="col-4 bg-host">
                                <div class="row text-right">
                                    <?php if (!empty($host['sla_log']['last_log_entry'])): ?>
                                        <?php if ($host['sla_log']['last_log_entry']['determined_availability_percent'] <
                                            $host['sla_log']['last_log_entry']['minimal_availability_percent']): ?>
                                            <div class="col-6 text-danger">
                                                <?= $host['sla_log']['last_log_entry']['determined_availability_percent']; ?>
                                                %
                                            </div>
                                            <div class="col-6">
                                                <div class="availability-status text-danger">
                                                    <?= __('Failed'); ?>
                                                </div>

                                            </div>
                                        <?php else: ?>
                                            <div class="col-6">
                                                <?= $host['sla_log']['last_log_entry']['determined_availability_percent']; ?>
                                                %
                                            </div>
                                            <div class="col-6">
                                                <div class="availability-status text-success">
                                                    <?= __('Passed'); ?>
                                                </div>
                                            </div>
                                        <?php endif; ?>
                                    <?php endif; ?>
                                </div>
                            </div>
                        </div>

                        <?php if (!empty($host['sla_log']['outages'])): ?>
                            <div class="row">
                                <div class="col-11 offset-1 padding-left-20 padding-top-10 text-danger bold">
                                    <?= __('Outages'); ?>:
                                </div>
                                <div class="col-2 offset-1 padding-left-20 bold">
                                    <?= __('Start'); ?>
                                </div>
                                <div class="col-2 bold">
                                    <?= __('End'); ?>
                                </div>
                                <div class="col-2 bold">
                                    <?= __('Duration'); ?>
                                </div>
                                <div class="col-5 bold">
                                    <?= __('Output'); ?>
                                </div>
                            </div>

                            <?php foreach ($host['sla_log']['outages'] as $outage): ?>
                                <div class="row">
                                    <div class="col-2 offset-1 padding-left-20">
                                        <?= $outage['start_human_readable']; ?>
                                    </div>
                                    <div class="col-2">
                                        <?= $outage['end_human_readable']; ?>
                                    </div>
                                    <div class="col-2">
                                        <?= $outage['total_time']; ?>
                                    </div>
                                    <div class="col-5 code-font">
                                        <?= h($outage['output']); ?>
                                    </div>
                                </div>
                            <?php endforeach; ?>
                        <?php endif; ?>
                        <?php if (!empty($host['sla_log']['outages_in_downtime'])): ?>
                            <div class="row">
                                <div class="col-11 offset-1 padding-left-20 padding-top-10 text-primary bold">
                                    <?= __('Outages in downtime'); ?>:
                                </div>
                                <div class="col-2 offset-1 padding-left-20 bold">
                                    <?= __('Start'); ?>
                                </div>
                                <div class="col-2 bold">
                                    <?= __('End'); ?>
                                </div>
                                <div class="col-2 bold">
                                    <?= __('Duration'); ?>
                                </div>
                                <div class="col-5 bold">
                                    <?= __('Output'); ?>
                                </div>
                            </div>

                            <?php foreach ($host['sla_log']['outages_in_downtime'] as $outage): ?>
                                <div class="row">
                                    <div class="col-2 offset-1 padding-left-20">
                                        <?= $outage['start_human_readable']; ?>
                                    </div>
                                    <div class="col-2">
                                        <?= $outage['end_human_readable']; ?>
                                    </div>
                                    <div class="col-2">
                                        <?= $outage['total_time']; ?>
                                    </div>
                                    <div class="col-5 code-font">
                                        <?= h($outage['output']); ?>
                                    </div>
                                </div>
                            <?php endforeach; ?>
                        <?php endif; ?>

                        <?php if (isset($host['services'])): ?>
                            <?php foreach ($host['services'] as $serviceIndex => $service): ?>
                                <div class="row break-inside-avoid">
                                    <div class="margin-top-5 col-8 padding-left-15 <?= ($serviceIndex % 2) ? 'bg-service' : ''; ?>">
                                        <i class="fa-solid fa-cog"></i>
                                        <?= h($service['service']['servicename']); ?>
                                    </div>
                                    <div class="margin-top-5 col-4  <?= ($serviceIndex % 2) ? 'bg-service' : ''; ?>">
                                        <div class="row text-right">
                                            <?php if (!empty($service['sla_log']['last_log_entry'])): ?>
                                                <?php if ($service['sla_log']['last_log_entry']['determined_availability_percent'] <
                                                    $service['sla_log']['last_log_entry']['minimal_availability_percent']): ?>
                                                    <div class="col-6 text-danger">
                                                        <?= $service['sla_log']['last_log_entry']['determined_availability_percent']; ?>
                                                        %
                                                    </div>
                                                    <div class="col-6">
                                                        <div class="availability-status text-danger">
                                                            <?= __('Failed'); ?>
                                                        </div>
                                                    </div>
                                                <?php else: ?>
                                                    <div class="col-6">
                                                        <?= $service['sla_log']['last_log_entry']['determined_availability_percent']; ?>
                                                        %
                                                    </div>
                                                    <div class="col-6">
                                                        <div class="availability-status text-success">
                                                            <?= __('Passed'); ?>
                                                        </div>
                                                    </div>
                                                <?php endif; ?>
                                            <?php endif; ?>
                                        </div>
                                    </div>
                                </div>

                                <?php if (!empty($service['sla_log']['outages'])): ?>
                                    <div class="row">
                                        <div class="col-11 offset-1 padding-left-20 padding-top-10 text-danger bold">
                                            <?= __('Outages'); ?>:
                                        </div>
                                        <div class="col-2 offset-1 padding-left-20 bold">
                                            <?= __('Start'); ?>
                                        </div>
                                        <div class="col-2 bold">
                                            <?= __('End'); ?>
                                        </div>
                                        <div class="col-2 bold">
                                            <?= __('Duration'); ?>
                                        </div>
                                        <div class="col-5 bold">
                                            <?= __('Output'); ?>
                                        </div>
                                    </div>

                                    <?php foreach ($service['sla_log']['outages'] as $outage): ?>
                                        <div class="row">
                                            <div class="col-2 offset-1 padding-left-20">
                                                <?= $outage['start_human_readable']; ?>
                                            </div>
                                            <div class="col-2">
                                                <?= $outage['end_human_readable']; ?>
                                            </div>
                                            <div class="col-2">
                                                <?= $outage['total_time']; ?>
                                            </div>
                                            <div class="col-5 code-font">
                                                <?= h($outage['output']); ?>
                                            </div>
                                        </div>
                                    <?php endforeach; ?>
                                <?php endif; ?>
                                <?php if (!empty($service['sla_log']['outages_in_downtime'])): ?>
                                    <div class="row">
                                        <div class="col-11 offset-1 padding-left-20 padding-top-10 text-primary bold">
                                            <?= __('Outages in downtime'); ?>:
                                        </div>
                                        <div class="col-2 offset-1 padding-left-20 bold">
                                            <?= __('Start'); ?>
                                        </div>
                                        <div class="col-2 bold">
                                            <?= __('End'); ?>
                                        </div>
                                        <div class="col-2 bold">
                                            <?= __('Duration'); ?>
                                        </div>
                                        <div class="col-5 bold">
                                            <?= __('Output'); ?>
                                        </div>
                                    </div>

                                    <?php foreach ($service['sla_log']['outages_in_downtime'] as $outage): ?>
                                        <div class="row">
                                            <div class="col-2 offset-1 padding-left-20">
                                                <?= $outage['start_human_readable']; ?>
                                            </div>
                                            <div class="col-2">
                                                <?= $outage['end_human_readable']; ?>
                                            </div>
                                            <div class="col-2">
                                                <?= $outage['total_time']; ?>
                                            </div>
                                            <div class="col-5 code-font">
                                                <?= h($outage['output']); ?>
                                            </div>
                                        </div>
                                    <?php endforeach; ?>
                                <?php endif; ?>
                            <?php endforeach; ?>
                        <?php endif; ?>
                    <?php endforeach; ?>
                </div>
            </div>
        <?php endforeach; ?>
    </div>
</div>
