<?php
declare(strict_types=1);

namespace SLAModule\Test\TestCase\Command;

use Cake\TestSuite\ConsoleIntegrationTestTrait;
use Cake\TestSuite\TestCase;
use itnovum\openITCOCKPIT\Core\Reports\DaterangesCreator;
use itnovum\openITCOCKPIT\Core\Reports\StatehistoryConverter;
use itnovum\openITCOCKPIT\Core\Views\Downtime;
use itnovum\openITCOCKPIT\Core\Views\StatehistoryService;
use SLAModule\Reports\SlaDaterangesCreator;

/**
 *
 * How to run Test
 * apt-get install phpunit php-sqlite3
 * cd /opt/openitc/frontend/plugins/SLAModule
 * composer install
 * phpunit tests/TestCase/Command/SlaDaterangesCreatorCommandTest.php
 * vendor/phpunit/phpunit/phpunit tests/TestCase/Command/SlaDaterangesCreatorCommandTest.php
 *
 * SLAModule\Command\SlaDaterangesCreatorCommand Test Case
 *
 * @uses \SLAModule\Command\SlaDaterangesCreatorCommand
 */
class SlaDaterangesCreatorCommandTest extends TestCase {
    use ConsoleIntegrationTestTrait;

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void {
        parent::setUp();
        $this->useCommandRunner();
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForToday() {
        $today = date('d.m.Y') . ' 00:01:00';
        $today = strtotime($today);

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $today,
            'DAY'
        );


        $assert = [
            0 => [
                'start'     => strtotime(date('d.m.Y') . ' 00:00:00'),
                'end'       => time(),
                'entryType' => 'new',
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForTwoDays() {
        $yesterday = strtotime('yesterday 00:10:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $yesterday,
            'DAY'
        );


        $assert = [
            0 => [
                'start'     => strtotime('yesterday 00:00:00'),
                'end'       => strtotime('today 00:00:00'),
                'entryType' => 'new'
            ],
            1 => [
                'start'     => strtotime('today 00:00:00'),
                'end'       => time(),
                'entryType' => 'new'
            ],
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForLastWeek() {
        $mondayLastWeek = strtotime('monday last week 05:10:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $mondayLastWeek
            , 'WEEK'
        );

        //print_r($dateRanges);
        //var_dump(date('d.m.Y H:i:s', $dateRanges[0]['start']));
        //var_dump(date('d.m.Y H:i:s', $dateRanges[0]['end']));
        //var_dump(date('d.m.Y H:i:s', $dateRanges[1]['start']));
        //var_dump(date('d.m.Y H:i:s', $dateRanges[1]['end']));

        $assert = [
            0 => [
                'start'     => strtotime('monday last week 00:00:00'),
                'end'       => strtotime('monday this week 00:00:00'),
                'entryType' => 'new'
            ],
            1 => [
                'start'     => strtotime('monday this week 00:00:00'),
                'end'       => time(),
                'entryType' => 'new'
            ],

        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForLastTwoWeeks() {
        // -3 weeks = two weeks ago
        $mondayTwotWeeksAgo = strtotime('monday -3 weeks 05:10:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $mondayTwotWeeksAgo,
            'WEEK'
        );


        $assert = [
            0 => [
                'start'     => strtotime('monday -3 weeks 00:00:00'),
                'end'       => strtotime('monday last week 00:00:00'),
                'entryType' => 'new'
            ],
            1 => [
                'start'     => strtotime('monday last week 00:00:00'),
                'end'       => strtotime('monday this week 00:00:00'),
                'entryType' => 'new'
            ],
            2 => [
                'start'     => strtotime('monday this week 00:00:00'),
                'end'       => time(),
                'entryType' => 'new'
            ],

        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForLastTwoMonths() {
        $twoMonthsAgo = strtotime('today -3 months 05:10:00');

        $fifteenthDayOfMonthThreeMonthsAgo = '15.' . date('m.Y H:i:s', $twoMonthsAgo);
        $fifteenthDayOfMonthThreeMonthsAgoTimestamp = strtotime($fifteenthDayOfMonthThreeMonthsAgo);

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $fifteenthDayOfMonthThreeMonthsAgoTimestamp,
            'MONTH'
        );

        //$this->debugDateRanges($dateRanges);

        $firstDayOfMonthTwoMonthsAgo = '01.' . date('m.Y H:i:s', strtotime('today -2 months 00:00:00'));
        $firstDayOfMonthLastMonths = '01.' . date('m.Y H:i:s', strtotime('today -1 month 00:00:00'));
        $firstDayOfMonthCurrentMonths = '01.' . date('m.Y') . '00:00:00';

        $assert = [
            0 => [
                'start'     => strtotime(date('d.m.Y', $fifteenthDayOfMonthThreeMonthsAgoTimestamp) . ' 00:00:00'),
                'end'       => strtotime($firstDayOfMonthTwoMonthsAgo),
                'entryType' => 'update'
            ],
            1 => [
                'start'     => strtotime($firstDayOfMonthTwoMonthsAgo),
                'end'       => strtotime($firstDayOfMonthLastMonths),
                'entryType' => 'new'
            ],
            2 => [
                'start'     => strtotime($firstDayOfMonthLastMonths),
                'end'       => strtotime($firstDayOfMonthCurrentMonths),
                'entryType' => 'new'
            ],
            3 => [
                'start'     => strtotime($firstDayOfMonthCurrentMonths),
                'end'       => time(),
                'entryType' => 'new'
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForLastTwoQuarters() {
        $fourMonthsAgo = strtotime('today -4 months 05:10:00');
        $fifthOfMonthFourMonthsAgo = strtotime('5 ' . date('F Y 13:00:00', $fourMonthsAgo));
        $quarter = SlaDaterangesCreator::getNumberFromQuarter($fifthOfMonthFourMonthsAgo);
        $quarterEnds = [1 => 'April', 2 => 'July', 3 => 'October', 4 => 'January'];

        $firstDayOfNextQuater = sprintf('1 %s %s 00:00:00',
            $quarterEnds[$quarter],
            $quarterEnds[$quarter] === 'January' ? intval(date('Y')) : date('Y')
        );

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $fifthOfMonthFourMonthsAgo,
            'QUARTER'
        );


        //$this->debugDate($fifthOfMonthFourMonthsAgo);
        //$this->debugDateRanges($dateRanges);
        //die();

        $assert = [
            0 => [
                'start'     => strtotime(date('d.m.Y', $fifthOfMonthFourMonthsAgo) . ' 00:00:00'),
                'end'       => strtotime($firstDayOfNextQuater),
                'entryType' => 'update'
            ],
            1 => [
                'start'     => strtotime($firstDayOfNextQuater),
                'end'       => time(),
                'entryType' => 'new'
            ]

        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateSlicesByIntervalAndLastUpdateDateForLastThreeYears() {
        $fifthOfJanuaryThreeYearsAgo = strtotime('5 January ' . (intval(date('Y')) - 3) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesForSlaByIntervalAndLastUpdateDate(
            $fifthOfJanuaryThreeYearsAgo,
            'YEAR'
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'start'     => strtotime('5 January ' . (intval(date('Y')) - 3) . ' 00:00:00'),
                'end'       => strtotime('1 January ' . (intval(date('Y')) - 2) . ' 00:00:00'),
                'entryType' => 'update'
            ],
            1 => [
                'start'     => strtotime('1 January ' . (intval(date('Y')) - 2) . ' 00:00:00'),
                'end'       => strtotime('1 January ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'entryType' => 'new'
            ],
            2 => [
                'start'     => strtotime('1 January ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'       => strtotime('1 January ' . date('Y') . ' 00:00:00'),
                'entryType' => 'new'
            ],
            3 => [
                'start'     => strtotime('1 January ' . date('Y') . ' 00:00:00'),
                'end'       => time(),
                'entryType' => 'new'
            ],

        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testSlaDaterangesCreatorCommandTestWith24x7Timerange() {
        $timerange = $this->getNaemonTimerange24x7();

        $start = strtotime('monday last week 00:00:00');
        $end = strtotime('monday this week 00:00:00'); // Workaround for "sunday last week 24:00:00"

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        //$this->debugDateRanges($dateRanges);

        $asset = [
            0 => [
                'start' => strtotime('Monday last week 00:00:00'),
                'end'   => strtotime('Tuesday last week 00:00:00'),
            ],
            1 => [
                'start' => strtotime('Tuesday last week 00:00:00'),
                'end'   => strtotime('Wednesday last week 00:00:00'),
            ],
            2 => [
                'start' => strtotime('Wednesday last week 00:00:00'),
                'end'   => strtotime('Thursday last week 00:00:00'),
            ],
            3 => [
                'start' => strtotime('Thursday last week 00:00:00'),
                'end'   => strtotime('Friday last week 00:00:00'),
            ],
            4 => [
                'start' => strtotime('Friday last week 00:00:00'),
                'end'   => strtotime('Saturday last week 00:00:00'),
            ],
            5 => [
                'start' => strtotime('Saturday last week 00:00:00'),
                'end'   => strtotime('Sunday last week 00:00:00'),
            ],
            6 => [
                'start' => strtotime('Sunday last week 00:00:00'),
                'end'   => strtotime('Monday this week 00:00:00'),
            ],
        ];

        $this->assertEquals($asset, $dateRanges);
    }

    public function test8x5timeperiods() {
        $timerange = $this->getNaemonTimerange8x5();

        $start = strtotime('monday last week 00:00:00');
        $end = strtotime('sunday last week'); // Workaround for "sunday last week 24:00:00"

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );


        $asset = [
            0 => [
                'start' => strtotime('Monday last week 10:00:00'),
                'end'   => strtotime('Monday last week 18:00:00'),
            ],
            1 => [
                'start' => strtotime('Tuesday last week 10:00:00'),
                'end'   => strtotime('Tuesday last week 18:00:00'),
            ],
            2 => [
                'start' => strtotime('Wednesday last week 10:00:00'),
                'end'   => strtotime('Wednesday last week 18:00:00'),
            ],
            3 => [
                'start' => strtotime('Thursday last week 10:00:00'),
                'end'   => strtotime('Thursday last week 18:00:00'),
            ],
            4 => [
                'start' => strtotime('Friday last week 10:00:00'),
                'end'   => strtotime('Friday last week 18:00:00'),
            ]
        ];

        $this->assertEquals($asset, $dateRanges);
    }

    public function testCustomtimeperiods() {
        $timerange = $this->getNaemonTimerangeCustom();

        $start = strtotime('monday last week 00:00:00');
        $end = strtotime('sunday last week'); // Workaround for "sunday last week 24:00:00"

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        //$this->debugDateRanges($dateRanges);

        $asset = [
            0 => [
                'start' => strtotime('Monday last week 08:00:00'),
                'end'   => strtotime('Monday last week 16:00:00'),
            ],
            1 => [
                'start' => strtotime('Monday last week 20:00:00'),
                'end'   => strtotime('Tuesday last week 00:00:00'),
            ],
            2 => [
                'start' => strtotime('Tuesday last week 08:00:00'),
                'end'   => strtotime('Tuesday last week 16:00:00'),
            ],
            3 => [
                'start' => strtotime('Tuesday last week 20:00:00'),
                'end'   => strtotime('Wednesday last week 00:00:00'),
            ]
        ];

        $this->assertEquals($asset, $dateRanges);
    }

    public function testCustomtimeperiodsSeptember2022() {
        $timerange = $this->getNaemonTimerangeCustom();

        $start = strtotime('15.09.2022 00:00:00');
        $end = strtotime('01.10.2022 00:00:00'); // Workaround for "sunday last week 24:00:00"

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        //$this->debugDateRanges($dateRanges);

        $asset = [
            0 => [
                'start' => strtotime('19.09.2022 08:00:00'),
                'end'   => strtotime('19.09.2022 16:00:00'),
            ],
            1 => [
                'start' => strtotime('19.09.2022 20:00:00'),
                'end'   => strtotime('20.09.2022 00:00:00'),
            ],
            2 => [
                'start' => strtotime('20.09.2022 08:00:00'),
                'end'   => strtotime('20.09.2022 16:00:00'),
            ],
            3 => [
                'start' => strtotime('20.09.2022 20:00:00'),
                'end'   => strtotime('21.09.2022 00:00:00'),
            ],
            4 => [
                'start' => strtotime('26.09.2022 08:00:00'),
                'end'   => strtotime('26.09.2022 16:00:00'),
            ],
            5 => [
                'start' => strtotime('26.09.2022 20:00:00'),
                'end'   => strtotime('27.09.2022 00:00:00'),
            ],
            6 => [
                'start' => strtotime('27.09.2022 08:00:00'),
                'end'   => strtotime('27.09.2022 16:00:00'),
            ],
            7 => [
                'start' => strtotime('27.09.2022 20:00:00'),
                'end'   => strtotime('28.09.2022 00:00:00'),
            ]
        ];

        $this->assertEquals($asset, $dateRanges);
    }


    public function testCustomtimeperiodsSeptember2022OneDayEmpty() {
        $timerange = $this->getNaemonTimerangeCustom();

        $start = strtotime('18.09.2022 00:00:00');
        $end = strtotime('18.09.2022 24:00:00'); // Workaround for "sunday last week 24:00:00"

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        //$this->debugDateRanges($dateRanges);

        $asset = [];

        $this->assertEquals($asset, $dateRanges);
    }


    public function testCustomtimeperiodsSeptember2022OneDayEmptyWednesday() {
        $timerange = $this->getNaemonTimerangeCustomWithThursday();

        $start = strtotime('21.09.2022 00:00:00');
        $end = strtotime('21.09.2022 24:00:00');

        $dateRanges = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        //$this->debugDateRanges($dateRanges);

        $asset = [];

        $this->assertEquals($asset, $dateRanges);
    }

    public function testServiceOutagesFromStatehistory24x7SoftAndHardState() {
        $statehistory = $this->getStatehistoryServiceRecords();
        $timerange = $this->getNaemonTimerange24x7();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $start = strtotime('17.05.2022 02:00:00');
        $end = strtotime('24.05.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            false,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 582805, //@todo verify this value
            1                                => 0,
            2                                => 21863,
            3                                => 132,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('18.05.2022 07:00:14'),
                    'end'          => strtotime('18.05.2022 13:00:32'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ],
                1 => [
                    'start'        => strtotime('23.05.2022 06:41:11'),
                    'end'          => strtotime('23.05.2022 06:45:16'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 21618
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testServiceOutagesFromStatehistory24x7OnlyHardState() {
        $statehistory = $this->getStatehistoryServiceRecords();
        $timerange = $this->getNaemonTimerange24x7();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $start = strtotime('17.05.2022 02:00:00');
        $end = strtotime('24.05.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 583080, //@todo verify this value
            1                                => 0,
            2                                => 21720,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('18.05.2022 07:01:25'),
                    'end'          => strtotime('18.05.2022 13:00:32'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ],
                1 => [
                    'start'        => strtotime('23.05.2022 06:42:23'),
                    'end'          => strtotime('23.05.2022 06:45:16'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 21547
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testServiceOutagesFromStatehistory8x5OnlyHardState() {
        $statehistory = $this->getStatehistoryServiceRecords();
        $timerange = $this->getNaemonTimerange8x5();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $start = strtotime('17.05.2022 02:00:00');
        $end = strtotime('24.05.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 133168, //@todo verify this value
            1                                => 0,
            2                                => 10832,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('18.05.2022 10:00:00'),
                    'end'          => strtotime('18.05.2022 13:00:32'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 10832
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testServiceOutagesFromStatehistory8x5OnlySoftAndHardState() {
        $statehistory = $this->getStatehistoryServiceRecords();
        $timerange = $this->getNaemonTimerange8x5();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $start = strtotime('17.05.2022 02:00:00');
        $end = strtotime('24.05.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            false,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 133168, //@todo verify this value
            1                                => 0,
            2                                => 10832,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('18.05.2022 10:00:00'),
                    'end'          => strtotime('18.05.2022 13:00:32'),
                    'output'       => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 10832
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testServiceOutagesFromStatehistoryCustomOnlyHardState() {
        $statehistory = $this->getStatehistoryServiceRecords();
        $timerange = $this->getNaemonTimerangeCustom();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $start = strtotime('17.05.2022 02:00:00');
        $end = strtotime('24.05.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        $assert = [
            0                                => 86400, //@todo verify this value
            1                                => 0,
            2                                => 0,
            3                                => 0,
            'outages'                        => [],
            'outages_summarized_in_downtime' => []
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testGetLongOutageForOneDaySLA24x7() {
        // Outage start: 04.08.2022 07:22:17: Outage end: none!
        $statehistory = $this->getAlwaysCriticalStatehistoryServiceRecord();
        $timerange = $this->getNaemonTimerange24x7();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        // SLA Report per day
        // Ok until 07:22:17. Critical after this timestamp
        $start = strtotime('04.08.2022 00:00:00');
        $end = strtotime('05.08.2022 00:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );
        //$this->debugDateRanges($timeSlices);

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );


        $assert = [
            0                                => 0,
            1                                => 0,
            2                                => 86400,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('04.08.2022 00:00:00'),
                    'end'          => strtotime('05.08.2022 00:00:00'),
                    'output'       => 'DISK CRIT DUMMY - free space: 5%:',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 86400
        ];

        $this->assertEquals($assert, $outages);

        // Second day
        // Critical for complete period

        $start = strtotime('05.08.2022 00:00:00');
        $end = strtotime('05.08.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 0,
            2                                => 86400,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('05.08.2022 00:00:00'),
                    'end'          => strtotime('05.08.2022 24:00:00'),
                    'output'       => 'DISK CRIT DUMMY - free space: 5%:',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 86400
        ];

        $this->assertEquals($assert, $outages);


        // Another day
        // Critical for complete period
        $start = strtotime('17.08.2022 00:00:00');
        $end = strtotime('17.08.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 0,
            2                                => 86400,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('17.08.2022 00:00:00'),
                    'end'          => strtotime('17.08.2022 24:00:00'),
                    'output'       => 'DISK CRIT DUMMY - free space: 5%:',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 86400
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testGetLongOutageForOneDaySLA8x5() {
        // Outage start: 04.08.2022 07:22:17: Outage end: none!
        $statehistory = $this->getAlwaysCriticalStatehistoryServiceRecord();
        $timerange = $this->getNaemonTimerange8x5();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        // SLA Report per day
        // Ok until 07:22:17. Critical after this timestamp
        $start = strtotime('04.08.2022 00:00:00');
        $end = strtotime('04.08.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        $assert = [
            0                                => 0,
            1                                => 0,
            2                                => 28800,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('04.08.2022 10:00:00'),
                    'end'          => strtotime('04.08.2022 18:00:00'),
                    'output'       => 'DISK CRIT DUMMY - free space: 5%:',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 28800
        ];

        $this->assertEquals($assert, $outages);

        // Second day
        // Critical for complete period
        $start = strtotime('05.08.2022 00:00:00');
        $end = strtotime('05.08.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 0,
            2                                => 28800,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('05.08.2022 10:00:00'),
                    'end'          => strtotime('05.08.2022 18:00:00'),
                    'output'       => 'DISK CRIT DUMMY - free space: 5%:',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 28800
        ];

        $this->assertEquals($assert, $outages);
    }

    public function testSLAOutagesPerDayRealWorldExample24x7() {
        // Statehistory data from 06.07.2022 14:08:45 to 04.11.2022 12:19:36
        // Outage in Statehistory 23.09.2022 07:00:21 to 26.09.2022 09:09:03
        $statehistory = $this->getRealServiceStatehistoryRecords();
        $timerange = $this->getNaemonTimerange24x7();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        // SLA day 1 => 23.09.2022
        $start = strtotime('23.09.2022 00:00:00');
        $end = strtotime('23.09.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );
        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 25221,
            2                                => 61179,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('23.09.2022 07:00:21'),
                    'end'          => strtotime('23.09.2022 24:00:00'),
                    'output'       => 'CRITICAL: Disk usage: 16.89GB/19.52GB (91.26%).',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 61179
        ];

        $this->assertEquals($assert, $outages);
        return;

        // SLA day 2 => 24.09.2022 and SLA day 3 => 25.09.2022
        // 100% outage
        for ($i = 24; $i <= 25; $i++) {
            $start = strtotime($i . '.09.2022 00:00:00'); //24.09 and 25.09
            $end = strtotime($i . '.09.2022 24:00:00');

            $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
                $start,
                $end,
                $timerange
            );

            $outages = StatehistoryConverter::generateReportDataWithOutages(
                $timeSlices,
                $statehistoryArray,
                true,
                false,
                true
            );

            //$this->debugDateRanges($outages['outages']);
            $assert = [
                0                                => 0,
                1                                => 0,
                2                                => 86400,
                3                                => 0,
                'outages'                        => [
                    0 => [
                        'start'        => strtotime($i . '.09.2022 00:00:00'),
                        'end'          => strtotime($i . '.09.2022 24:00:00'),
                        'output'       => 'CRITICAL: Disk usage: 16.89GB/19.52GB (91.26%).',
                        'is_hardstate' => true,
                        'is_downtime'  => false
                    ]
                ],
                'outages_summarized_in_downtime' => [],
                'maximum_outage_duration'        => 86400
            ];

            $this->assertEquals($assert, $outages);
        }

        // SLA day 4 => 26.09.2022
        $start = strtotime('26.09.2022 00:00:00');
        $end = strtotime('26.09.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 60657,
            2                                => 25743,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('26.09.2022 00:00:00'),
                    'end'          => strtotime('26.09.2022 07:09:03'),
                    'output'       => 'CRITICAL: Disk usage: 16.89GB/19.52GB (91.26%).',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 25743
        ];
        $this->assertEquals($assert, $outages);

        // SLA day 5 => 27.09.2022
        $start = strtotime('27.09.2022 00:00:00');
        $end = strtotime('27.09.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        //$this->debugDateRanges($outages['outages']);

        $assert = [
            0                                => 0,
            1                                => 40624,
            2                                => 45776,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('27.09.2022 11:17:04'),
                    'end'          => strtotime('27.09.2022 24:00:00'),
                    'output'       => 'CRITICAL: Disk usage: 16.65GB/19.52GB (90%).',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [],
            'maximum_outage_duration'        => 45776
        ];
        $this->assertEquals($assert, $outages);
    }

    public function testSLAOutagesPerDayRealWorldExample24x7AndDowntime() {
        // Statehistory data from 06.07.2022 14:08:45 to 04.11.2022 12:19:36
        // Outage in Statehistory 23.09.2022 07:00:21 to 26.09.2022 09:09:03
        // Outage in Statehistory 27.09.2022 11:17:04 to 02.11.2022 09:07:12
        // Downtime: 27.09.2022 16:00:00 to 14.11.2022 13:35:00
        $statehistory = $this->getRealServiceStatehistoryRecords();
        $timerange = $this->getNaemonTimerange24x7();

        $statehistoryArray = [];
        foreach ($statehistory as $statehistoryService) {
            $statehistoryArray[] = $statehistoryService->toArray();
        }

        $downtimes = [];
        foreach ($this->getDowntimeForRealServiceStatehistoryRecords() as $d) {
            $endTime = $d->getScheduledEndTime();
            if ($d->wasCancelled()) {
                $endTime = $d->getActualEndTime();
            }

            $downtimes[] = [
                'scheduled_start_time' => date('Y-m-d H:i:s', $d->getScheduledStartTime()),
                'scheduled_end_time'   => date('Y-m-d H:i:s', $endTime),
                'duration'             => $d->getDuration(),
                'was_started'          => (int)$d->wasStarted(),
                'was_cancelled'        => (int)$d->wasCancelled(),
            ];
        }

        // SLA day 1 => 27.09.2022
        $start = strtotime('27.09.2022 00:00:00');
        $end = strtotime('27.09.2022 24:00:00');

        $timeSlices = SlaDaterangesCreator::createDateRangesForSla(
            $start,
            $end,
            $timerange
        );

        $downtimesFiltered = DaterangesCreator::mergeTimeOverlapping(
            array_map(
                function ($downtimes) {
                    return [
                        'start_time' => strtotime($downtimes['scheduled_start_time']),
                        'end_time'   => strtotime($downtimes['scheduled_end_time']),
                    ];
                },
                $downtimes
            )
        );

        $timeSlices = DaterangesCreator::setDowntimesInTimeslices(
            $timeSlices,
            $downtimesFiltered
        );


        $outages = StatehistoryConverter::generateReportDataWithOutages(
            $timeSlices,
            $statehistoryArray,
            true,
            false,
            true
        );

        $assert = [
            0                                => 28800,
            1                                => 40624,
            2                                => 16976,
            3                                => 0,
            'outages'                        => [
                0 => [
                    'start'        => strtotime('27.09.2022 11:17:04'),
                    'end'          => strtotime('27.09.2022 16:00:00'),
                    'output'       => 'CRITICAL: Disk usage: 16.65GB/19.52GB (90%).',
                    'is_hardstate' => true,
                    'is_downtime'  => false
                ]
            ],
            'outages_summarized_in_downtime' => [
                0 => [
                    'start'        => strtotime('27.09.2022 16:00:00'),
                    'end'          => strtotime('27.09.2022 24:00:00'),
                    'output'       => 'CRITICAL: Disk usage: 16.65GB/19.52GB (90%).',
                    'is_hardstate' => true,
                    'is_downtime'  => true
                ]
            ],
            'maximum_outage_duration'        => 16976
        ];
        $this->assertEquals($assert, $outages);

    }

    public function testCreateDateRangesByIntervalDayAndStartDateAndEndDate1Day() {
        $startDate = strtotime('31 December ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('31 December ' . (intval(date('Y')) - 1) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'DAY',
            $startDate,
            $endDate
        );
        $weekdays = $this->getWeekdays();

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => $weekdays[date('N', strtotime('31 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))],
                'start' => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }


    public function testCreateDateRangesByIntervalDayAndStartDateAndEndDate6Days() {
        $startDate = strtotime('29 December ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('3 January ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'DAY',
            $startDate,
            $endDate
        );
        $weekdays = $this->getWeekdays();

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => $weekdays[date('N', strtotime('29 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))],
                'start' => strtotime('29 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('29 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            1 => [
                'title' => $weekdays[date('N', strtotime('30 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))],
                'start' => strtotime('30 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('30 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            2 => [
                'title' => $weekdays[date('N', strtotime('31 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))],
                'start' => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            3 => [
                'title' => $weekdays[date('N', strtotime('1 January ' . date('Y') . ' 00:00:00'))],
                'start' => strtotime('1 January ' . date('Y') . ' 00:00:00'),
                'end'   => strtotime('1 January ' . date('Y') . ' 24:00:00')
            ],
            4 => [
                'title' => $weekdays[date('N', strtotime('2 January ' . date('Y') . ' 00:00:00'))],
                'start' => strtotime('2 January ' . date('Y') . ' 00:00:00'),
                'end'   => strtotime('2 January ' . date('Y') . ' 24:00:00')
            ],
            5 => [
                'title' => $weekdays[date('N', strtotime('3 January ' . date('Y') . ' 00:00:00'))],
                'start' => strtotime('3 January ' . date('Y') . ' 00:00:00'),
                'end'   => strtotime('3 January ' . date('Y') . ' 24:00:00')
            ],
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalWeekAndStartDateAndEndDate4Weeks() {
        $startDate = strtotime('21 December ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('8 January ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'WEEK',
            $startDate,
            $endDate
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => __('Week {0}', date('W [Y]', strtotime('21 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('21 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('25 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            1 => [
                'title' => __('Week {0}', date('W [Y]', strtotime('26 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('26 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('01 January ' . (intval(date('Y'))) . ' 24:00:00')
            ],
            2 => [
                'title' => __('Week {0}', date('W [Y]', strtotime('02 January ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('02 January ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('08 January ' . (intval(date('Y'))) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalMonthAndStartDateAndEndDate4Months() {
        $startDate = strtotime('21 November ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('25 February ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'MONTH',
            $startDate,
            $endDate
        );
        $months = $this->getMonths();

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('%s [%s]', $months[11], date('Y', strtotime('21 November ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('21 November ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('30 November ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            1 => [
                'title' => sprintf('%s [%s]', $months[12], date('Y', strtotime('01 December ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 December ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            2 => [
                'title' => sprintf('%s [%s]', $months[1], date('Y', strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('31 January ' . (intval(date('Y'))) . ' 24:00:00')
            ],
            3 => [
                'title' => sprintf('%s [%s]', $months[2], date('Y', strtotime('01 February ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('01 February ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('25 February ' . (intval(date('Y'))) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalQuarterAndStartDateAndEndDate6Quarters() {
        $startDate = strtotime('21 November ' . (intval(date('Y')) - 2) . ' 13:00:00');
        $endDate = strtotime('25 February ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'QUARTER',
            $startDate,
            $endDate
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('4 Quarter [%s]', date('Y', strtotime('21 November ' . (intval(date('Y')) - 2) . ' 00:00:00'))),
                'start' => strtotime('21 November ' . (intval(date('Y')) - 2) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 2) . ' 24:00:00')
            ],
            1 => [
                'title' => sprintf('1 Quarter [%s]', date('Y', strtotime('01 January ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 January ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 March ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            2 => [
                'title' => sprintf('2 Quarter [%s]', date('Y', strtotime('01 April ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 April ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('30 June ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            3 => [
                'title' => sprintf('3 Quarter [%s]', date('Y', strtotime('01 July ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 July ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('30 September ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            4 => [
                'title' => sprintf('4 Quarter [%s]', date('Y', strtotime('01 October ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 October ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            5 => [
                'title' => sprintf('1 Quarter [%s]', date('Y', strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('25 February ' . (intval(date('Y'))) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalQuarterAndStartDateAndEndDateHalfQuarter() {
        $startDate = strtotime('8 February ' . (intval(date('Y'))) . ' 13:00:00');
        $endDate = strtotime('25 February ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'QUARTER',
            $startDate,
            $endDate
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('1 Quarter [%s]', date('Y', strtotime('8 February  ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('8 February  ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('25 February ' . (intval(date('Y'))) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalYearAndStartDateAndEndDate2Years() {
        $startDate = strtotime('28 February ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('25 February ' . (intval(date('Y'))) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'YEAR',
            $startDate,
            $endDate
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('Year %s', date('Y', strtotime('28 February ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('28 February ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ],
            1 => [
                'title' => sprintf('Year %s', date('Y', strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'))),
                'start' => strtotime('01 January ' . (intval(date('Y'))) . ' 00:00:00'),
                'end'   => strtotime('25 February ' . (intval(date('Y'))) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }

    public function testCreateDateRangesByIntervalYearAndStartDateAndEndDate1Year() {
        $startDate = strtotime('01 January ' . (intval(date('Y')) - 1) . ' 13:00:00');
        $endDate = strtotime('31 December ' . (intval(date('Y')) - 1) . ' 13:00:00');

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'YEAR',
            $startDate,
            $endDate
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('Year %s', date('Y', strtotime('01 January ' . (intval(date('Y')) - 1) . ' 00:00:00'))),
                'start' => strtotime('01 January ' . (intval(date('Y')) - 1) . ' 00:00:00'),
                'end'   => strtotime('31 December ' . (intval(date('Y')) - 1) . ' 24:00:00')
            ]
        ];

        $this->assertEquals($assert, $dateRanges);
    }


    public function testCreateDateRangesByIntervalAndStartDateAndEndDateToday() {
        $startDate = strtotime(date('d.m.Y', time()) . ' 05:00:00');
        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'DAY',
            $startDate,
            time()
        );
        $weekdays = $this->getWeekdays();

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => $weekdays[date('N', strtotime('yesterday 00:00:00'))],
                'start' => strtotime('yesterday 00:00:00'),
                'end'   => strtotime('today 00:00:00'),
            ]
        ];

        $this->assertEquals($assert, $dateRanges);

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'WEEK',
            $startDate,
            time()
        );

        //$this->debugDateRanges($dateRanges);


        $assert = [
            0 => [
                'title' => __('Week {0}', date('W [Y]', strtotime('yesterday 00:00:00'))),
                'start' => strtotime('yesterday 00:00:00'),
                'end'   => strtotime('today 00:00:00'),
            ]
        ];

        $this->assertEquals($assert, $dateRanges);

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'MONTH',
            $startDate,
            time()
        );

        $months = $this->getMonths();


        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('%s [%s]', $months[date('n', strtotime('yesterday 00:00:00'))], date('Y', strtotime('yesterday 00:00:00'))),
                'start' => strtotime('yesterday 00:00:00'),
                'end'   => strtotime('today 00:00:00'),
            ]
        ];

        $this->assertEquals($assert, $dateRanges);

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'QUARTER',
            $startDate,
            time()
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('%s Quarter [%s]', ceil(date('n', strtotime('yesterday 00:00:00')) / 3), date('Y', strtotime('yesterday 00:00:00'))),
                'start' => strtotime('yesterday 00:00:00'),
                'end'   => strtotime('today 00:00:00'),
            ]
        ];

        $this->assertEquals($assert, $dateRanges);

        $dateRanges = SlaDaterangesCreator::createDateSlicesByIntervalAndStartAndEndDate(
            'YEAR',
            $startDate,
            time()
        );

        //$this->debugDateRanges($dateRanges);

        $assert = [
            0 => [
                'title' => sprintf('Year %s', date('Y', strtotime('yesterday 00:00:00'))),
                'start' => strtotime('yesterday 00:00:00'),
                'end'   => strtotime('today 00:00:00'),
            ]
        ];

        $this->assertEquals($assert, $dateRanges);

    }


    /***********************************
     *       TEST HELPER METHODS       *
     ***********************************/

    private function getNaemonTimerange24x7() {
        return [
            (int)0 => [
                'id'            => (int)1,
                'timeperiod_id' => (int)1,
                'day'           => (int)1,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)1 => [
                'id'            => (int)2,
                'timeperiod_id' => (int)1,
                'day'           => (int)2,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)2 => [
                'id'            => (int)3,
                'timeperiod_id' => (int)1,
                'day'           => (int)3,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)3 => [
                'id'            => (int)4,
                'timeperiod_id' => (int)1,
                'day'           => (int)4,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)4 => [
                'id'            => (int)5,
                'timeperiod_id' => (int)1,
                'day'           => (int)5,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)5 => [
                'id'            => (int)6,
                'timeperiod_id' => (int)1,
                'day'           => (int)6,
                'start'         => '00:00',
                'end'           => '24:00'
            ],
            (int)6 => [
                'id'            => (int)7,
                'timeperiod_id' => (int)1,
                'day'           => (int)7,
                'start'         => '00:00',
                'end'           => '24:00'
            ]
        ];
    }

    private function getNaemonTimerange8x5() {
        return [
            (int)0 => [
                'id'            => (int)12,
                'timeperiod_id' => (int)4,
                'day'           => (int)1,
                'start'         => '10:00',
                'end'           => '18:00'
            ],
            (int)1 => [
                'id'            => (int)13,
                'timeperiod_id' => (int)4,
                'day'           => (int)2,
                'start'         => '10:00',
                'end'           => '18:00'
            ],
            (int)2 => [
                'id'            => (int)14,
                'timeperiod_id' => (int)4,
                'day'           => (int)3,
                'start'         => '10:00',
                'end'           => '18:00'
            ],
            (int)3 => [
                'id'            => (int)15,
                'timeperiod_id' => (int)4,
                'day'           => (int)4,
                'start'         => '10:00',
                'end'           => '18:00'
            ],
            (int)4 => [
                'id'            => (int)16,
                'timeperiod_id' => (int)4,
                'day'           => (int)5,
                'start'         => '10:00',
                'end'           => '18:00'
            ]
        ];
    }

    private function getNaemonTimerangeCustom() {
        return [
            (int)0 => [
                'id'            => (int)17,
                'timeperiod_id' => (int)5,
                'day'           => (int)1, // Monday
                'start'         => '08:00',
                'end'           => '16:00'
            ],
            (int)1 => [
                'id'            => (int)18,
                'timeperiod_id' => (int)5,
                'day'           => (int)1, // Monday
                'start'         => '20:00',
                'end'           => '24:00'
            ],
            (int)2 => [
                'id'            => (int)19,
                'timeperiod_id' => (int)5,
                'day'           => (int)2, // Tuesday
                'start'         => '08:00',
                'end'           => '16:00'
            ],
            (int)3 => [
                'id'            => (int)20,
                'timeperiod_id' => (int)5,
                'day'           => (int)2, // Tuesday
                'start'         => '20:00',
                'end'           => '24:00'
            ]
        ];
    }


    private function getNaemonTimerangeCustomWithThursday() {
        return [
            (int)0 => [
                'id'            => (int)17,
                'timeperiod_id' => (int)5,
                'day'           => (int)1, // Monday
                'start'         => '08:00',
                'end'           => '16:00'
            ],
            (int)1 => [
                'id'            => (int)18,
                'timeperiod_id' => (int)5,
                'day'           => (int)1, // Monday
                'start'         => '20:00',
                'end'           => '24:00'
            ],
            (int)2 => [
                'id'            => (int)19,
                'timeperiod_id' => (int)5,
                'day'           => (int)2, // Tuesday
                'start'         => '08:00',
                'end'           => '16:00'
            ],
            (int)3 => [
                'id'            => (int)20,
                'timeperiod_id' => (int)5,
                'day'           => (int)2, // Tuesday
                'start'         => '20:00',
                'end'           => '24:00'
            ],
            (int)4 => [
                'id'            => (int)20,
                'timeperiod_id' => (int)5,
                'day'           => (int)4, // Thursday
                'start'         => '20:00',
                'end'           => '24:00'
            ]
        ];
    }

    /**
     * @return StatehistoryService[]
     */
    private function getStatehistoryServiceRecords() {
        $dbRecords = [
            (int)0 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1652768636,
                'state_time_usec'       => (int)936347,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)3,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)1,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)0,
                'last_hard_state'       => (int)0,
                'output'                => 'UNKNOWN:  Get "http://127.0.0.1:1234/runHostcheck?hostuuid=6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9": context deadline exceeded (Client.Timeout exceeded while awaiting headers)',
                'long_output'           => 'UNKNOWN:  Get "http://127.0.0.1:1234/runHostcheck?hostuuid=6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9": context deadline exceeded (Client.Timeout exceeded while awaiting headers)'
            ],
            (int)1 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1652768702,
                'state_time_usec'       => (int)401678,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)0,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 4.7 sec',
                'long_output'           => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 4.7 sec'
            ],
            (int)2 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1652857214,
                'state_time_usec'       => (int)355610,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)1,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)0,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                'long_output'           => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec'
            ],
            (int)3 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1652857285,
                'state_time_usec'       => (int)833732,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                'long_output'           => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec'
            ],
            (int)4 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1652878832,
                'state_time_usec'       => (int)171712,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)0,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 7.9 sec',
                'long_output'           => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 7.9 sec'
            ],
            (int)5 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1653288071,
                'state_time_usec'       => (int)497422,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)1,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)0,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                'long_output'           => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec'
            ],
            (int)6 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1653288143,
                'state_time_usec'       => (int)340316,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec',
                'long_output'           => '[agent] Communication failed: timed out(!!), [snmp] Cannot fetch system description OID .1.3.6.1.2.1.1.1.0. Please check your SNMP configuration. Possible reason might be: Wrong credentials, wrong SNMP version, Firewall rules, etc.(!!), Got no information from host, execution time 11.0 sec'
            ],
            (int)7 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1653288316,
                'state_time_usec'       => (int)992185,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)0,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 5.1 sec',
                'long_output'           => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 5.1 sec'
            ],
            (int)8 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1653288647,
                'state_time_usec'       => (int)494532,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)3,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)1,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)0,
                'last_hard_state'       => (int)0,
                'output'                => 'UNKNOWN:  Get "http://127.0.0.1:1234/runHostcheck?hostuuid=6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9": context deadline exceeded (Client.Timeout exceeded while awaiting headers)',
                'long_output'           => 'UNKNOWN:  Get "http://127.0.0.1:1234/runHostcheck?hostuuid=6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9": context deadline exceeded (Client.Timeout exceeded while awaiting headers)'
            ],
            (int)9 => [
                'service_description'   => '1bdb5ca7-66e2-4045-89cf-32a144bd5112',
                'state_time'            => (int)1653288713,
                'state_time_usec'       => (int)96175,
                'hostname'              => '6d6c8ff6-0efd-4116-a1c2-9b6a18813bd9',
                'state_change'          => true,
                'state'                 => (int)0,
                'is_hardstate'          => false,
                'current_check_attempt' => (int)2,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)0,
                'output'                => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 4.9 sec',
                'long_output'           => '[agent] Version: 1.6.0p11, OS: linux, [snmp] Success, execution time 4.9 sec'
            ]
        ];

        $valueObjects = [];
        foreach ($dbRecords as $record) {
            $valueObjects[] = new StatehistoryService($record);
        }

        return $valueObjects;
    }

    /**
     * @return StatehistoryService[]
     */
    private function getAlwaysCriticalStatehistoryServiceRecord() {
        return [
            new StatehistoryService([
                'service_description'   => '1b69c940-5d9a-4bb5-948c-eef3645cf3ce',
                'state_time'            => (int)1659555000, // 03.08.2022 19:30:00
                'state_time_usec'       => (int)936347,
                'hostname'              => 'c36b8048-93ce-4385-ac19-ab5c90574b77',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)1,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)0,
                'last_hard_state'       => (int)0,
                'output'                => 'DISK CRIT DUMMY - free space: 5%:',
                'long_output'           => 'DISK CRIT DUMMY - free space: 5%:'
            ])
        ];
    }

    /**
     * This has data from
     * 09.06.2022 07:31:30 until 24.06.2022 11:39:28
     * @return StatehistoryService[]
     */
    private function getRealServiceStatehistoryRecords() {
        $dbRecords = [
            (int)0  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1657116525, // 06.07.2022 14:08:45
                'state_time_usec'       => (int)522200,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)1,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)3,
                'output'                => 'WARNING: Disk usage: 15.05GB/19.56GB (81.13%).',
                'long_output'           => 'WARNING: Disk usage: 15.05GB/19.56GB (81.13%).'
            ],
            (int)1  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1662619701, //08.09.2022 06:48:21
                'state_time_usec'       => (int)990508,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)3,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)1,
                'last_hard_state'       => (int)1,
                'output'                => 'UNKNOWN: State is no longer current',
                'long_output'           => 'UNKNOWN: State is no longer current'
            ],
            (int)2  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1662701444, //09.09.2022 05:30:44
                'state_time_usec'       => (int)696688,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)1,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)3,
                'output'                => 'WARNING: Disk usage: 15.89GB/19.52GB (85.87%).',
                'long_output'           => 'WARNING: Disk usage: 15.89GB/19.52GB (85.87%).'
            ],
            (int)3  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1663916421, //23.09.2022 07:00:21
                'state_time_usec'       => (int)777803,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)1,
                'last_hard_state'       => (int)1,
                'output'                => 'CRITICAL: Disk usage: 16.89GB/19.52GB (91.26%).',
                'long_output'           => 'CRITICAL: Disk usage: 16.89GB/19.52GB (91.26%).'
            ],
            (int)4  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1664176143, //26.09.2022 07:09:03
                'state_time_usec'       => (int)982172,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)1,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => 'WARNING: Disk usage: 16.56GB/19.52GB (89.52%).',
                'long_output'           => 'WARNING: Disk usage: 16.56GB/19.52GB (89.52%).'
            ],
            (int)5  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1664277424, //27.09.2022 11:17:04
                'state_time_usec'       => (int)452967,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)1,
                'last_hard_state'       => (int)1,
                'output'                => 'CRITICAL: Disk usage: 16.65GB/19.52GB (90%).',
                'long_output'           => 'CRITICAL: Disk usage: 16.65GB/19.52GB (90%).'
            ],
            (int)6  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1667383632, //02.11.2022 10:07:12
                'state_time_usec'       => (int)436578,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)3,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => 'UNKNOWN: State is no longer current',
                'long_output'           => 'UNKNOWN: State is no longer current'
            ],
            (int)7  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1667389776, //02.11.2022 11:49:36
                'state_time_usec'       => (int)202822,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)3,
                'output'                => 'CRITICAL: Disk usage: 18.46GB/19.52GB (99.77%).',
                'long_output'           => 'CRITICAL: Disk usage: 18.46GB/19.52GB (99.77%).'
            ],
            (int)8  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1667400316, //02.11.2022 14:45:16
                'state_time_usec'       => (int)575537,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)3,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => 'UNKNOWN: No data received from agent',
                'long_output'           => 'UNKNOWN: No data received from agent'
            ],
            (int)9  => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1667400636, //02.11.2022 14:50:36
                'state_time_usec'       => (int)336533,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)2,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)3,
                'last_hard_state'       => (int)3,
                'output'                => 'CRITICAL: Disk usage: 18.43GB/19.52GB (99.62%).',
                'long_output'           => 'CRITICAL: Disk usage: 18.43GB/19.52GB (99.62%).'
            ],
            (int)10 => [
                'service_description'   => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'state_time'            => (int)1667564376, //04.11.2022 12:19:36
                'state_time_usec'       => (int)754687,
                'hostname'              => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'state_change'          => true,
                'state'                 => (int)0,
                'is_hardstate'          => true,
                'current_check_attempt' => (int)3,
                'max_check_attempts'    => (int)3,
                'last_state'            => (int)2,
                'last_hard_state'       => (int)2,
                'output'                => 'OK: Disk usage: 12.05GB/19.52GB (65.1%).',
                'long_output'           => 'OK: Disk usage: 12.05GB/19.52GB (65.1%).'
            ]
        ];

        $valueObjects = [];
        foreach ($dbRecords as $record) {
            $valueObjects[] = new StatehistoryService($record);
        }

        return $valueObjects;
    }

    private function getDowntimeForRealServiceStatehistoryRecords() {
        return [
            new Downtime([
                'hostname'             => '9e3c497c-a206-4ee5-a4b1-3629d3667593',
                'service_description'  => '9bb6a848-6e8d-4769-b644-08b373b587f9',
                'internal_downtime_id' => (int)5116,
                'scheduled_start_time' => (int)1664294400, // 27.09.2022 16:00:00
                'node_name'            => 'openITCOCKPIT',
                'entry_time'           => (int)1667568056, // 04.11.2022 13:20:56
                'entry_time_usec'      => (int)791613,
                'author_name'          => 'Root Wheel',
                'comment_data'         => 'In progress Downtime',
                'triggered_by_id'      => (int)0,
                'is_fixed'             => true,
                'duration'             => (int)4138500,
                'scheduled_end_time'   => (int)1668432900, // 14.11.2022 13:35:00
                'was_started'          => true,
                'actual_start_time'    => (int)1667568056, // 04.11.2022 13:20:56
                'actual_end_time'      => (int)0,
                'was_cancelled'        => false
            ])
        ];
    }

    private function debugDateRanges(array $dateRanges) {
        echo PHP_EOL;
        foreach ($dateRanges as $range) {
            if (isset($range['entryType'])) {
                // Timerange
                printf('Start: %s  End: %s  entryType: %s%s',
                    date('l d.m.Y H:i:s', $range['start']),
                    date('l d.m.Y H:i:s', $range['end']),
                    $range['entryType'],
                    PHP_EOL
                );
            } else {
                // Timeslice (24x7, 8x5)
                printf('Start: %s  End: %s %s',
                    date('l d.m.Y H:i:s', $range['start']),
                    date('l d.m.Y H:i:s', $range['end']),
                    PHP_EOL
                );
            }
        }
    }

    private function debugDate($date) {
        if (is_numeric($date)) {
            echo date('d.m.Y H:i:s', $date);
        } else {
            echo $date;
        }
        echo PHP_EOL;
    }

    private function getWeekdays() {
        return [
            1 => __('Monday'),
            2 => __('Tuesday'),
            3 => __('Wednesday'),
            4 => __('Thursday'),
            5 => __('Friday'),
            6 => __('Saturday'),
            7 => __('Sunday'),
        ];
    }

    private function getMonths() {
        return [
            1  => __('January'),
            2  => __('February'),
            3  => __('March'),
            4  => __('April'),
            5  => __('May'),
            6  => __('June'),
            7  => __('July'),
            8  => __('August'),
            9  => __('September'),
            10 => __('October'),
            11 => __('November'),
            12 => __('December')
        ];
    }
}
