angular.module('openITCOCKPIT')
    .controller('HostMappingRulesController', function($scope, $http, $state, $stateParams, NotyService, RedirectService, SortService, QueryStringService){

        $scope.id = $stateParams.id;
        $scope.init = true;
        $scope.hasError = null;
        $scope.errors = {};
        $scope.loadHostsSpinner = false;
        $scope.currentPage = 1;

        $scope.post = {
            host_keywords: null,
            host_not_keywords: null,
            hostname_regex: '',
            description: '',
            save_options: false,
            sla_id: $scope.id,
            only_unassigned: false
        };

        SortService.setSort(QueryStringService.getStateValue($stateParams, 'sort', 'Hosts.name'));
        SortService.setDirection(QueryStringService.getStateValue($stateParams, 'direction', 'asc'));

        $scope.load = function(){
            $http.get("/sla_module/host_mapping_rules/assignToHosts/" + $scope.id + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.sla = result.data.sla;
                if(result.data.sla.host_mapping_rule !== null){
                    $scope.post = result.data.sla.host_mapping_rule;
                }

                jQuery(function(){
                    $('.tagsinput').tagsinput();
                });

                setTimeout(function(){
                    $scope.init = false;
                }, 250);

            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };

        $scope.loadHosts = function(){
            $scope.loadHostsSpinner = true;
            $http.get("/sla_module/host_mapping_rules/loadHostsBySlaId.json", {
                params: {
                    'angular': true,
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'slaId': $scope.sla.id,
                    'filter[Hosts.name]': $scope.post.hostname_regex,
                    'filter[hostdescription]': $scope.post.description,
                    'filter[Hosts.keywords][]': ($scope.post.host_keywords !== null ? $scope.post.host_keywords.split(',') : []),
                    'filter[Hosts.not_keywords][]': ($scope.post.host_not_keywords !== null ? $scope.post.host_not_keywords.split(',') : []),
                    'resolveContainerIds': true,
                    'onlyUnassigned': $scope.post.only_unassigned
                }
            }).then(function(result){
                $scope.hosts = result.data.hosts;
                $scope.loadHostsSpinner = false;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;

            }, function errorCallback(result){
                NotyService.genericError();
                $scope.loadHostsSpinner = false;
            });
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.loadHosts();
            }
        };


        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.loadHosts();
        };

        $scope.submit = function(saveHostMappingRule){
            $scope.post.save_options = saveHostMappingRule || false;
            $scope.post.filter = {
                'Hosts.name': $scope.post.hostname_regex,
                'Hosts.keywords': ($scope.post.host_keywords !== null ? $scope.post.host_keywords.split(',') : []),
                'Hosts.not_keywords': ($scope.post.host_not_keywords !== null ? $scope.post.host_not_keywords.split(',') : []),
            };
            $http.post("/sla_module/host_mapping_rules/assignToHosts/" + $scope.id + ".json?angular=true",
                $scope.post
            ).then(function(result){
                NotyService.genericSuccess();
                RedirectService.redirectWithFallback('SlasIndex');
                $scope.errors = {};
            }, function errorCallback(result){
                if(result.data.hasOwnProperty('error')){
                    NotyService.genericError();
                    $scope.errors = result.data.error;
                }
            });
        };

        //Fire on page load
        $scope.load();
        SortService.setCallback($scope.loadHosts);

        $scope.$watch('post', function(){
            if($scope.init === false && $scope.hosts){
                $scope.currentPage = 1;
                $scope.loadHosts();
            }
        }, true);

    });
