angular.module('openITCOCKPIT')
    .controller('SlasViewDetailsController', function($scope, $http, $stateParams, $state, SortService){
        $scope.id = $stateParams.id;

        SortService.setSort('determined_availability');
        SortService.setDirection('asc');

        $scope.currentPage = 1;
        $scope.useScroll = true;

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                Hosts: {
                    name: '',
                    description: ''
                },
                determined_availability: {
                    from: null,
                    to: null
                }
            };
        };
        /*** Filter end ***/
        $scope.init = true;
        $scope.showFilter = false;

        $scope.load = function(){
            var params = {
                'angular': true,
                'scroll': $scope.useScroll,
                'sort': SortService.getSort(),
                'page': $scope.currentPage,
                'direction': SortService.getDirection(),
                'filter[Hosts.name]': $scope.filter.Hosts.name,
                'filter[determined_availability][]': [
                    $scope.filter.determined_availability.from || 0,
                    $scope.filter.determined_availability.to || 100
                ]
            };

            $http.get("/sla_module/slas/viewDetails/" + $scope.id + ".json", {
                params: params
            }).then(function(result){
                    $scope.sla = result.data.sla;
                    $scope.sla.weekDays = [];
                    for(var i = 0; i <= 6; i++){
                        // Array required to keep order
                        $scope.sla.weekDays[i] = [];
                    }
                    for(var key in $scope.sla.timeperiod.timeperiod_timeranges){
                        var timeRange = $scope.sla.timeperiod.timeperiod_timeranges[key];
                        // Server day 1 - 7
                        // JavaScript days 0 - 6 to get an array (array has to start with 0!) :(
                        var day = timeRange.day - 1;
                        $scope.sla.weekDays[day].push({
                            start: timeRange.start,
                            end: timeRange.end
                        });
                    }
                    $scope.availability = result.data.availability;
                    $scope.slaDetails = result.data.slaDetails;
                    $scope.paging = result.data.paging;
                    $scope.scroll = result.data.scroll;
                    $scope.init = false;

                },
                function errorCallback(result){
                    if(result.status === 403){
                        $state.go('403');
                    }

                    if(result.status === 404){
                        $state.go('404');
                    }
                }
            );
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        //Fire on page load
        defaultFilter();
        SortService.setCallback($scope.load);

        $scope.$watch('filter', function(){
            $scope.currentPage = 1;
            $scope.load();
        }, true);
    });
